@@ -1,25 +1,16 @@
 /*
- * "$Id: parallel.c 5726 2006-07-12 20:00:11Z mike $"
+ * "$Id: parallel.c 7021 2007-10-10 22:50:07Z mike $"
  *
  *   Parallel port backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
  *   "LICENSE" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -27,6 +18,7 @@
  *
  *   main()         - Send a file to the specified parallel port.
  *   list_devices() - List all parallel devices.
+ *   side_cb()      - Handle side-channel requests...
  */
 
 /*
@@ -66,7 +58,8 @@
  * Local functions...
  */
 
-void	list_devices(void);
+static void	list_devices(void);
+static void	side_cb(int print_fd, int device_fd, int use_bc);
 
 
 /*
@@ -88,7 +81,8 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 		*options;		/* Pointer to options */
   int		port;			/* Port number (not used) */
   int		print_fd,		/* Print file */
-		device_fd;		/* Parallel device */
+		device_fd,		/* Parallel device */
+		use_bc;			/* Read back-channel data? */
   int		copies;			/* Number of copies to print */
   size_t	tbytes;			/* Total number of bytes written */
   struct termios opts;			/* Parallel port options */
@@ -128,7 +122,9 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   }
   else if (argc < 6 || argc > 7)
   {
-    fputs("Usage: parallel job-id user title copies options [file]\n", stderr);
+    _cupsLangPrintf(stderr,
+		    _("Usage: %s job-id user title copies options [file]\n"),
+		    argv[0]);
     return (CUPS_BACKEND_FAILED);
   }
 
@@ -188,7 +184,26 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 
   do
   {
-    if ((device_fd = open(resource, O_WRONLY | O_EXCL)) == -1)
+#if defined(__linux) || defined(__FreeBSD__)
+   /*
+    * The Linux and FreeBSD parallel port drivers currently are broken WRT
+    * select() and bidirection I/O...
+    */
+
+    device_fd = open(resource, O_WRONLY | O_EXCL);
+    use_bc    = 0;
+
+#else
+    if ((device_fd = open(resource, O_RDWR | O_EXCL)) < 0)
+    {
+      device_fd = open(resource, O_WRONLY | O_EXCL);
+      use_bc    = 0;
+    }
+    else
+      use_bc = 1;
+#endif /* __linux || __FreeBSD__ */
+
+    if (device_fd == -1)
     {
       if (getenv("CLASS") != NULL)
       {
@@ -199,8 +214,9 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 	* available printer in the class.
 	*/
 
-        fputs("INFO: Unable to open parallel port, queuing on next printer "
-	      "in class...\n", stderr);
+        _cupsLangPuts(stderr,
+	              _("INFO: Unable to contact printer, queuing on next "
+			"printer in class...\n"));
 
        /*
         * Sleep 5 seconds to keep the job from requeuing too rapidly...
@@ -213,21 +229,22 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 
       if (errno == EBUSY)
       {
-        fputs("INFO: Parallel port busy; will retry in 30 seconds...\n",
-	      stderr);
+        _cupsLangPuts(stderr,
+	              _("INFO: Printer busy; will retry in 30 seconds...\n"));
 	sleep(30);
       }
       else if (errno == ENXIO || errno == EIO || errno == ENOENT)
       {
-        fputs("INFO: Printer not connected; will retry in 30 seconds...\n",
-	      stderr);
+        _cupsLangPuts(stderr,
+	              _("INFO: Printer not connected; will retry in 30 "
+		        "seconds...\n"));
 	sleep(30);
       }
       else
       {
-	fprintf(stderr,
-	        "ERROR: Unable to open parallel port device file \"%s\": %s\n",
-	        resource, strerror(errno));
+	_cupsLangPrintf(stderr,
+		        _("ERROR: Unable to open device file \"%s\": %s\n"),
+			resource, strerror(errno));
 	return (CUPS_BACKEND_FAILED);
       }
     }
@@ -264,11 +281,16 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
       lseek(print_fd, 0, SEEK_SET);
     }
 
-    tbytes = backendRunLoop(print_fd, device_fd, 1);
+    tbytes = backendRunLoop(print_fd, device_fd, use_bc, side_cb);
 
     if (print_fd != 0 && tbytes >= 0)
-      fprintf(stderr, "INFO: Sent print file, " CUPS_LLFMT " bytes...\n",
-	      CUPS_LLCAST tbytes);
+      _cupsLangPrintf(stderr,
+#ifdef HAVE_LONG_LONG
+		      _("INFO: Sent print file, %lld bytes...\n"),
+#else
+		      _("INFO: Sent print file, %ld bytes...\n"),
+#endif /* HAVE_LONG_LONG */
+		      CUPS_LLCAST tbytes);
   }
 
  /*
@@ -288,7 +310,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
  * 'list_devices()' - List all parallel devices.
  */
 
-void
+static void
 list_devices(void)
 {
 #if defined(__hpux) || defined(__sgi) || defined(__sun)
@@ -302,7 +324,8 @@ list_devices(void)
   char	device[255],		/* Device filename */
 	basedevice[255],	/* Base device filename for ports */
 	device_id[1024],	/* Device ID string */
-	make_model[1024];	/* Make and model */
+	make_model[1024],	/* Make and model */
+	uri[1024];		/* Device URI */
 
 
   if (!access("/dev/parallel/", 0))
@@ -328,13 +351,15 @@ list_devices(void)
       * Now grab the IEEE 1284 device ID string...
       */
 
+      snprintf(uri, sizeof(uri), "parallel:%s", device);
+
       if (!backendGetDeviceID(fd, device_id, sizeof(device_id),
                               make_model, sizeof(make_model),
-			      NULL, NULL, 0))
-	printf("direct parallel:%s \"%s\" \"%s LPT #%d\" \"%s\"\n", device,
+			      NULL, uri, sizeof(uri)))
+	printf("direct %s \"%s\" \"%s LPT #%d\" \"%s\"\n", uri,
 	       make_model, make_model, i + 1, device_id);
       else
-	printf("direct parallel:%s \"Unknown\" \"LPT #%d\"\n", device, i + 1);
+	printf("direct %s \"Unknown\" \"LPT #%d\"\n", uri, i + 1);
 
       close(fd);
     }
@@ -578,5 +603,72 @@ list_devices(void)
 
 
 /*
- * End of "$Id: parallel.c 5726 2006-07-12 20:00:11Z mike $".
+ * 'side_cb()' - Handle side-channel requests...
+ */
+
+static void
+side_cb(int print_fd,			/* I - Print file */
+        int device_fd,			/* I - Device file */
+	int use_bc)			/* I - Using back-channel? */
+{
+  cups_sc_command_t	command;	/* Request command */
+  cups_sc_status_t	status;		/* Request/response status */
+  char			data[2048];	/* Request/response data */
+  int			datalen;	/* Request/response data size */
+
+
+  datalen = sizeof(data);
+
+  if (cupsSideChannelRead(&command, &status, data, &datalen, 1.0))
+  {
+    _cupsLangPuts(stderr, _("WARNING: Failed to read side-channel request!\n"));
+    return;
+  }
+
+  switch (command)
+  {
+    case CUPS_SC_CMD_DRAIN_OUTPUT :
+        if (backendDrainOutput(print_fd, device_fd))
+	  status = CUPS_SC_STATUS_IO_ERROR;
+	else if (tcdrain(device_fd))
+	  status = CUPS_SC_STATUS_IO_ERROR;
+	else
+	  status = CUPS_SC_STATUS_OK;
+
+	datalen = 0;
+        break;
+
+    case CUPS_SC_CMD_GET_BIDI :
+        data[0] = use_bc;
+        datalen = 1;
+        break;
+
+    case CUPS_SC_CMD_GET_DEVICE_ID :
+        memset(data, 0, sizeof(data));
+
+        if (backendGetDeviceID(device_fd, data, sizeof(data) - 1,
+	                       NULL, 0, NULL, NULL, 0))
+        {
+	  status  = CUPS_SC_STATUS_NOT_IMPLEMENTED;
+	  datalen = 0;
+	}
+	else
+        {
+	  status  = CUPS_SC_STATUS_OK;
+	  datalen = strlen(data);
+	}
+        break;
+
+    default :
+        status  = CUPS_SC_STATUS_NOT_IMPLEMENTED;
+	datalen = 0;
+	break;
+  }
+
+  cupsSideChannelWrite(command, status, data, datalen, 1.0);
+}
+
+
+/*
+ * End of "$Id: parallel.c 7021 2007-10-10 22:50:07Z mike $".
  */
