@@ -1,8 +1,9 @@
 /*
- * "$Id: scsi.c 5023 2006-01-29 14:39:44Z mike $"
+ * "$Id: scsi.c 6835 2007-08-22 18:34:34Z mike $"
  *
  *   SCSI printer backend for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 2003-2006 by Easy Software Products, all rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or
@@ -55,10 +56,12 @@
 
 #include <cups/backend.h>
 #include <cups/cups.h>
+#include <cups/i18n.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <cups/string.h>
+#include <cups/i18n.h>
 #include <signal.h>
 
 #ifdef WIN32
@@ -80,7 +83,7 @@ void	list_devices(void);
 int	print_device(const char *resource, int fd, int copies);
 
 
-#ifdef __linux__
+#if defined(__linux__) && defined(HAVE_SCSI_SG_H)
 #  include "scsi-linux.c"
 #elif defined(__sgi)
 #  include "scsi-irix.c"
@@ -90,7 +93,7 @@ int	print_device(const char *resource, int fd, int copies);
  */
 void	list_devices(void) {}
 int	print_device(const char *resource, int fd, int copies) { return (CUPS_BACKEND_FAILED); }
-#endif /* __linux */
+#endif /* __linux && HAVE_SCSI_SG_H */
 
 
 /*
@@ -150,7 +153,9 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   }
   else if (argc < 6 || argc > 7)
   {
-    fputs("Usage: scsi:/dev/file job-id user title copies options [file]\n", stderr);
+    _cupsLangPrintf(stderr,
+                    _("Usage: %s job-id user title copies options [file]\n"),
+		    argv[0]);
     return (CUPS_BACKEND_FAILED);
   }
 
@@ -220,5 +225,5 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
 
 /*
- * End of "$Id: scsi.c 5023 2006-01-29 14:39:44Z mike $".
+ * End of "$Id: scsi.c 6835 2007-08-22 18:34:34Z mike $".
  */
