@@ -1,25 +1,16 @@
 /*
- * "$Id: usb.c 6441 2007-04-04 13:54:51Z mike $"
+ * "$Id: usb.c 6911 2007-09-04 20:35:08Z mike $"
  *
  *   USB port backend for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
  *   "LICENSE" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -45,6 +36,7 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <cups/string.h>
+#include <cups/i18n.h>
 #include <signal.h>
 
 #ifdef WIN32
@@ -60,10 +52,10 @@
  * Local functions...
  */
 
-static void	list_devices(void);
-static int	print_device(const char *uri, const char *hostname,
-		             const char *resource, const char *options,
-			     int print_fd, int copies, int argc, char *argv[]);
+void	list_devices(void);
+int	print_device(const char *uri, const char *hostname,
+	             const char *resource, char *options,
+		     int print_fd, int copies, int argc, char *argv[]);
 
 
 /*
@@ -85,7 +77,7 @@ static int	print_device(const char *uri, const char *hostname,
  * 'list_devices()' - List all available USB devices to stdout.
  */
 
-static void
+void
 list_devices(void)
 {
  /*
@@ -103,11 +95,11 @@ list_devices(void)
  * 'print_device()' - Print a file to a USB device.
  */
 
-static int				/* O - Exit status */
+int					/* O - Exit status */
 print_device(const char *uri,		/* I - Device URI */
              const char *hostname,	/* I - Hostname/manufacturer */
              const char *resource,	/* I - Resource/modelname */
-	     const char *options,	/* I - Device options/serial number */
+	     char       *options,	/* I - Device options/serial number */
 	     int        print_fd,	/* I - File descriptor to print */
 	     int        copies,		/* I - Copies to print */
 	     int	argc,		/* I - Number of command-line arguments (6 or 7) */
@@ -192,7 +184,9 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   }
   else if (argc < 6 || argc > 7)
   {
-    fputs("Usage: usb job-id user title copies options [file]\n", stderr);
+    _cupsLangPrintf(stderr,
+                    _("Usage: %s job-id user title copies options [file]\n"),
+                    argv[0]);
     return (CUPS_BACKEND_FAILED);
   }
 
@@ -207,7 +201,9 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 		      hostname, sizeof(hostname), &port,
 		      resource, sizeof(resource)) < HTTP_URI_OK)
   {
-    fputs("ERROR: No device URI found in argv[0] or in DEVICE_URI environment variable!\n", stderr);
+    _cupsLangPuts(stderr,
+                  _("ERROR: No device URI found in argv[0] or in DEVICE_URI "
+	            "environment variable!\n"));
     return (1);
   }
 
@@ -243,8 +239,8 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 
     if ((print_fd = open(argv[6], O_RDONLY)) < 0)
     {
-      fprintf(stderr, "ERROR: unable to open print file %s - %s\n",
-              argv[6], strerror(errno));
+      _cupsLangPrintf(stderr, _("ERROR: Unable to open print file %s - %s\n"),
+                      argv[6], strerror(errno));
       return (CUPS_BACKEND_FAILED);
     }
 
@@ -270,5 +266,5 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 
 
 /*
- * End of "$Id: usb.c 6441 2007-04-04 13:54:51Z mike $".
+ * End of "$Id: usb.c 6911 2007-09-04 20:35:08Z mike $".
  */
