@@ -1,5 +1,5 @@
 /*
- * "$Id: file.c 6313 2007-02-27 15:46:10Z mike $"
+ * "$Id: file.c 6963 2007-09-17 20:35:54Z mike $"
  *
  *   File functions for the Common UNIX Printing System (CUPS).
  *
@@ -8,23 +8,14 @@
  *   our own file functions allows us to provide transparent support of
  *   gzip'd print files, PPD files, etc.
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -1216,7 +1207,7 @@ cupsFileRead(cups_file_t *fp,		/* I - CUPS file */
     if (fp->ptr >= fp->end)
       if (cups_fill(fp) <= 0)
       {
-        DEBUG_printf(("    cups_fill() returned -1, total=%d\n", total));
+        DEBUG_printf(("    cups_fill() returned -1, total=%d\n", (int)total));
 
         if (total > 0)
           return ((ssize_t)total);
@@ -1244,7 +1235,7 @@ cupsFileRead(cups_file_t *fp,		/* I - CUPS file */
   * Return the total number of bytes read...
   */
 
-  DEBUG_printf(("    total=%d\n", total));
+  DEBUG_printf(("    total=%d\n", (int)total));
 
   return ((ssize_t)total);
 }
@@ -1712,6 +1703,7 @@ cups_fill(cups_file_t *fp)		/* I - CUPS file */
 {
   ssize_t		bytes;		/* Number of bytes read */
 #ifdef HAVE_LIBZ
+  int			status;		/* Decompression status */
   const unsigned char	*ptr,		/* Pointer into buffer */
 			*end;		/* End of buffer */
 #endif /* HAVE_LIBZ */
@@ -1930,7 +1922,13 @@ cups_fill(cups_file_t *fp)		/* I - CUPS file */
       fp->stream.next_out  = (Bytef *)fp->buf;
       fp->stream.avail_out = sizeof(fp->buf);
 
-      if (inflate(&(fp->stream), Z_NO_FLUSH) == Z_STREAM_END)
+      status = inflate(&(fp->stream), Z_NO_FLUSH);
+
+      if (fp->stream.next_out > (Bytef *)fp->buf)
+        fp->crc = crc32(fp->crc, (Bytef *)fp->buf,
+	                fp->stream.next_out - (Bytef *)fp->buf);
+
+      if (status == Z_STREAM_END)
       {
        /*
 	* Read the CRC and length...
@@ -1959,6 +1957,9 @@ cups_fill(cups_file_t *fp)		/* I - CUPS file */
             * Bad CRC, mark end-of-file...
 	    */
 
+            DEBUG_printf(("cups_fill: tcrc=%08x, fp->crc=%08x\n",
+	                  (unsigned int)tcrc, (unsigned int)fp->crc));
+
 	    fp->eof = 1;
 
 	    return (-1);
@@ -2135,5 +2136,5 @@ cups_write(cups_file_t *fp,		/* I - CUPS file */
 
 
 /*
- * End of "$Id: file.c 6313 2007-02-27 15:46:10Z mike $".
+ * End of "$Id: file.c 6963 2007-09-17 20:35:54Z mike $".
  */
