@@ -1,25 +1,16 @@
 /*
- * "$Id: getputfile.c 5235 2006-03-06 13:02:23Z mike $"
+ * "$Id: getputfile.c 6720 2007-07-25 00:40:03Z mike $"
  *
  *   Get/put file functions for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -35,10 +26,9 @@
  * Include necessary headers...
  */
 
+#include "globals.h"
 #include "cups.h"
-#include "ipp.h"
 #include "language.h"
-#include "string.h"
 #include "debug.h"
 #include <stdlib.h>
 #include <ctype.h>
@@ -176,7 +166,10 @@ cupsGetFd(http_t     *http,		/* I - HTTP connection to server */
       write(fd, buffer, bytes);
   }
   else
+  {
+    _cupsSetHTTPError(status);
     httpFlush(http);
+  }
 
  /*
   * Return the request status...
@@ -425,7 +418,10 @@ cupsPutFd(http_t     *http,		/* I - HTTP connection to server */
   */
 
   if (status != HTTP_CREATED)
+  {
+    _cupsSetHTTPError(status);
     httpFlush(http);
+  }
 
   return (status);
 }
@@ -488,5 +484,5 @@ cupsPutFile(http_t     *http,		/* I - HTTP connection to server */
 
 
 /*
- * End of "$Id: getputfile.c 5235 2006-03-06 13:02:23Z mike $".
+ * End of "$Id: getputfile.c 6720 2007-07-25 00:40:03Z mike $".
  */
