@@ -1,5 +1,5 @@
 /*
- * "$Id: http-support.c 7030 2007-10-19 01:23:05Z mike $"
+ * "$Id: http-support.c 7319 2008-02-15 23:26:51Z mike $"
  *
  *   HTTP support routines for the Common UNIX Printing System (CUPS) scheduler.
  *
@@ -55,7 +55,8 @@
 
 static const char * const http_days[7] =
 			{
-			  "Sun",			  "Mon",
+			  "Sun",
+			  "Mon",
 			  "Tue",
 			  "Wed",
 			  "Thu",
@@ -1282,7 +1283,7 @@ http_copy_encode(char       *dst,	/* O - Destination buffer */
 		 const char *term,	/* I - Terminating characters */
 		 int        encode)	/* I - %-encode reserved chars? */
 {
-  static const char *hex = "0123456789ABCDEF";
+  static const char hex[] = "0123456789ABCDEF";
 
 
   while (*src && dst < dstend)
@@ -1318,5 +1319,5 @@ http_copy_encode(char       *dst,	/* O - Destination buffer */
 
 
 /*
- * End of "$Id: http-support.c 7030 2007-10-19 01:23:05Z mike $".
+ * End of "$Id: http-support.c 7319 2008-02-15 23:26:51Z mike $".
  */
