@@ -1,26 +1,17 @@
 /*
- * "$Id: http.h 6188 2007-01-10 16:23:06Z mike $"
+ * "$Id: http.h 6722 2007-07-25 17:19:09Z mike $"
  *
  *   Hyper-Text Transport Protocol definitions for the Common UNIX Printing
  *   System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  */
@@ -64,10 +55,11 @@ typedef off_t ssize_t;			/* @private@ */
 #    ifdef AF_LOCAL
 #      include <sys/un.h>
 #    endif /* AF_LOCAL */
+#    if defined(LOCAL_PEERCRED) && !defined(SO_PEERCRED)
+#      define SO_PEERCRED LOCAL_PEERCRED
+#    endif /* LOCAL_PEERCRED && !SO_PEERCRED */
 #  endif /* WIN32 */
 
-#  include "md5.h"
-
 /*
  * With GCC 3.0 and higher, we can mark old APIs "deprecated" so you get
  * a warning at compile-time.
@@ -141,7 +133,8 @@ typedef enum http_auth_e		/**** HTTP authentication types ****/
   HTTP_AUTH_MD5,			/* Digest authentication in use */
   HTTP_AUTH_MD5_SESS,			/* MD5-session authentication in use */
   HTTP_AUTH_MD5_INT,			/* Digest authentication in use for body */
-  HTTP_AUTH_MD5_SESS_INT		/* MD5-session authentication in use for body */
+  HTTP_AUTH_MD5_SESS_INT,		/* MD5-session authentication in use for body */
+  HTTP_AUTH_NEGOTIATE			/* GSSAPI authentication in use @since CUPS 1.3@ */
 } http_auth_t;
 
 typedef enum http_encoding_e		/**** HTTP transfer encoding values ****/
@@ -329,60 +322,7 @@ typedef struct http_addrlist_s		/**** Socket address list, which is
   http_addr_t		addr;		/* Address */
 } http_addrlist_t;
 
-typedef struct _http_s			/**** HTTP connection structure. ****/
-{
- /*
-  * DO NOT ACCESS MEMBERS OF THIS STRUCTURE DIRECTLY; INSTEAD, USE THE
-  * PROVIDED APIS FOR ACCESSING THE VALUES INSTEAD.
-  *
-  * This structure definition will be removed from the public headers in
-  * CUPS 1.3.
-  */
-
-  int			fd;		/* File descriptor for this socket */
-  int			blocking;	/* To block or not to block */
-  int			error;		/* Last error on read */
-  time_t		activity;	/* Time since last read/write */
-  http_state_t		state;		/* State of client */
-  http_status_t		status;		/* Status of last request */
-  http_version_t	version;	/* Protocol version */
-  http_keepalive_t	keep_alive;	/* Keep-alive supported? */
-  struct sockaddr_in	_hostaddr;	/* Address of connected host @deprecated@ */
-  char			hostname[HTTP_MAX_HOST],
-  					/* Name of connected host */
-			fields[HTTP_FIELD_MAX][HTTP_MAX_VALUE];
-					/* Field values */
-  char			*data;		/* Pointer to data buffer */
-  http_encoding_t	data_encoding;	/* Chunked or not */
-  int			_data_remaining;/* Number of bytes left @deprecated@ */
-  int			used;		/* Number of bytes used in buffer */
-  char			buffer[HTTP_MAX_BUFFER];
-					/* Buffer for incoming data */
-  int			auth_type;	/* Authentication in use */
-  _cups_md5_state_t	md5_state;	/* MD5 state */
-  char			nonce[HTTP_MAX_VALUE];
-					/* Nonce value */
-  int			nonce_count;	/* Nonce count */
-  void			*tls;		/* TLS state information */
-  http_encryption_t	encryption;	/* Encryption requirements */
-  /**** New in CUPS 1.1.19 ****/
-  fd_set		*input_set;	/* select() set for httpWait() @since CUPS 1.1.19@ */
-  http_status_t		expect;		/* Expect: header @since CUPS 1.1.19@ */
-  char			*cookie;	/* Cookie value(s) @since CUPS 1.1.19@ */
-  /**** New in CUPS 1.1.20 ****/
-  char			authstring[HTTP_MAX_VALUE],
-					/* Current Authentication value @since CUPS 1.1.20@ */
-			userpass[HTTP_MAX_VALUE];
-					/* Username:password string @since CUPS 1.1.20@ */
-  int			digest_tries;	/* Number of tries for digest auth @since CUPS 1.1.20@ */
-  /**** New in CUPS 1.2 ****/
-  off_t			data_remaining;	/* Number of bytes left @since CUPS 1.2@ */
-  http_addr_t		*hostaddr;	/* Current host address and port @since CUPS 1.2@ */
-  http_addrlist_t	*addrlist;	/* List of valid addresses @since CUPS 1.2@ */
-  char			wbuffer[HTTP_MAX_BUFFER];
-					/* Buffer for outgoing data */
-  int			wused;		/* Write buffer bytes used @since CUPS 1.2@ */
-} http_t;
+typedef struct _http_s http_t;		/**** HTTP connection type ****/
 
 
 /*
@@ -502,6 +442,10 @@ extern void		httpSetLength(http_t *http, size_t length);
 extern ssize_t		httpWrite2(http_t *http, const char *buffer,
 			           size_t length);
 
+/**** New in CUPS 1.3 ****/
+extern char		*httpGetAuthString(http_t *http);
+extern void		httpSetAuthString(http_t *http, const char *scheme,
+			                  const char *data);
 
 /*
  * C++ magic...
@@ -513,5 +457,5 @@ extern ssize_t		httpWrite2(http_t *http, const char *buffer,
 #endif /* !_CUPS_HTTP_H_ */
 
 /*
- * End of "$Id: http.h 6188 2007-01-10 16:23:06Z mike $".
+ * End of "$Id: http.h 6722 2007-07-25 17:19:09Z mike $".
  */
