@@ -1,26 +1,17 @@
 /*
- * "$Id: ipp.h 5878 2006-08-24 15:55:42Z mike $"
+ * "$Id: ipp.h 7050 2007-10-31 18:35:56Z mike $"
  *
  *   Internet Printing Protocol definitions for the Common UNIX Printing
  *   System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  */
@@ -64,7 +55,8 @@ extern "C" {
  * Common limits...
  */
 
-#  define IPP_MAX_NAME		256
+#  define IPP_MAX_LENGTH	32767	/* Maximum size of any single value */
+#  define IPP_MAX_NAME		256	/* Maximum length of common name values */
 #  define IPP_MAX_VALUES	8	/* Power-of-2 allocation increment */
 
 
@@ -254,7 +246,8 @@ typedef enum ipp_op_e			/**** IPP operations... ****/
   CUPS_GET_DEVICES,			/* Get a list of supported devices */
   CUPS_GET_PPDS,			/* Get a list of supported drivers */
   CUPS_MOVE_JOB,			/* Move a job to a different printer */
-  CUPS_AUTHENTICATE_JOB			/* Authenticate a job @since CUPS 1.2@ */
+  CUPS_AUTHENTICATE_JOB,		/* Authenticate a job @since CUPS 1.2@ */
+  CUPS_GET_PPD				/* Get a PPD file @since CUPS 1.3@ */
 } ipp_op_t;
 
 /* Old names for the operations */
@@ -271,7 +264,8 @@ typedef enum ipp_status_e		/**** IPP status codes... ****/
   IPP_OK_TOO_MANY_EVENTS,		/* successful-ok-too-many-events */
   IPP_OK_BUT_CANCEL_SUBSCRIPTION,	/* successful-ok-but-cancel-subscription */
   IPP_OK_EVENTS_COMPLETE,		/* successful-ok-events-complete */
-  IPP_REDIRECTION_OTHER_SITE = 0x300,	/*  */
+  IPP_REDIRECTION_OTHER_SITE = 0x200,	/*  */
+  CUPS_SEE_OTHER = 0x280,		/* cups-see-other */
   IPP_BAD_REQUEST = 0x0400,		/* client-error-bad-request */
   IPP_FORBIDDEN,			/* client-error-forbidden */
   IPP_NOT_AUTHENTICATED,		/* client-error-not-authenticated */
@@ -499,5 +493,5 @@ extern ipp_state_t	ippWriteIO(void *dst, ipp_iocb_t cb, int blocking,
 #endif /* !_CUPS_IPP_H_ */
 
 /*
- * End of "$Id: ipp.h 5878 2006-08-24 15:55:42Z mike $".
+ * End of "$Id: ipp.h 7050 2007-10-31 18:35:56Z mike $".
  */
