@@ -1,25 +1,16 @@
 /*
- * "$Id: md5passwd.c 5232 2006-03-05 17:59:19Z mike $"
+ * "$Id: md5passwd.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   MD5 password support for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -35,7 +26,7 @@
  * Include necessary headers...
  */
 
-#include "http.h"
+#include "http-private.h"
 #include "string.h"
 
 
@@ -147,5 +138,5 @@ httpMD5String(const unsigned char *sum,	/* I - MD5 sum data */
 
 
 /*
- * End of "$Id: md5passwd.c 5232 2006-03-05 17:59:19Z mike $".
+ * End of "$Id: md5passwd.c 6649 2007-07-11 21:46:42Z mike $".
  */
