@@ -1,25 +1,16 @@
 /*
- * "$Id: options.c 6313 2007-02-27 15:46:10Z mike $"
+ * "$Id: options.c 7262 2008-01-28 23:10:10Z mike $"
  *
  *   Option routines for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -31,6 +22,8 @@
  *   cupsMarkOptions()   - Mark command-line options in a PPD file.
  *   cupsParseOptions()  - Parse options from a command-line argument.
  *   cupsRemoveOptions() - Remove an option from an option array.
+ *   debug_marked()      - Output the marked array to stdout...
+ *   ppd_mark_choices()  - Mark one or more option choices from a string.
  */
 
 /*
@@ -44,6 +37,18 @@
 #include "debug.h"
 
 
+/*
+ * Local functions...
+ */
+
+#ifdef DEBUG
+static void	debug_marked(ppd_file_t *ppd, const char *title);
+#else
+#  define debug_marked(ppd,title)
+#endif /* DEBUG */
+static int	ppd_mark_choices(ppd_file_t *ppd, const char *options);
+
+
 /*
  * 'cupsAddOption()' - Add an option to an option array.
  */
@@ -171,6 +176,7 @@ cupsMarkOptions(
   const char	*page_size;		/* PageSize option */
   cups_option_t	*optptr;		/* Current option */
   ppd_option_t	*option;		/* PPD option */
+  ppd_attr_t	*attr;			/* PPD attribute */
   static const char * const duplex_options[] =
 		{			/* Duplex option names */
 		  "Duplex",		/* Adobe */
@@ -205,6 +211,8 @@ cupsMarkOptions(
   if (ppd == NULL || num_options <= 0 || options == NULL)
     return (0);
 
+  debug_marked(ppd, "Before...");
+
  /*
   * Mark options...
   */
@@ -256,7 +264,7 @@ cupsMarkOptions(
             conflict = 1;
 
         if (cupsGetOption("EFMediaType", num_options, options) == NULL)
-	  if (ppdMarkOption(ppd, "EFMediaType", s))
+	  if (ppdMarkOption(ppd, "EFMediaType", s))		/* EFI */
             conflict = 1;
 
         if (cupsGetOption("EFMediaQualityMode", num_options, options) == NULL)
@@ -366,13 +374,79 @@ cupsMarkOptions(
     }
     else if (!strcasecmp(optptr->name, "output-bin"))
     {
-      if (cupsGetOption("OutputBin", num_options, options) == NULL)
+      if (!cupsGetOption("OutputBin", num_options, options))
         if (ppdMarkOption(ppd, "OutputBin", optptr->value))
           conflict = 1;
     }
+    else if (!strcasecmp(optptr->name, "multiple-document-handling"))
+    {
+      if (!cupsGetOption("Collate", num_options, options) &&
+          ppdFindOption(ppd, "Collate"))
+      {
+        if (strcasecmp(optptr->value, "separate-documents-uncollated-copies"))
+	{
+	  if (ppdMarkOption(ppd, "Collate", "True"))
+            conflict = 1;
+        }
+	else
+	{
+	  if (ppdMarkOption(ppd, "Collate", "False"))
+            conflict = 1;
+        }
+      }
+    }
+    else if (!strcasecmp(optptr->name, "finishings"))
+    {
+     /*
+      * Lookup cupsIPPFinishings attributes for each value...
+      */
+
+      for (ptr = optptr->value; *ptr;)
+      {
+       /*
+        * Get the next finishings number...
+	*/
+
+        if (!isdigit(*ptr & 255))
+	  break;
+
+        if ((j = strtol(ptr, &ptr, 10)) < 3)
+	  break;
+
+       /*
+        * Skip separator as needed...
+	*/
+
+        if (*ptr == ',')
+	  ptr ++;
+
+       /*
+        * Look it up in the PPD file...
+	*/
+
+	sprintf(s, "%d", j);
+
+        if ((attr = ppdFindAttr(ppd, "cupsIPPFinishings", s)) == NULL)
+	  continue;
+
+       /*
+        * Apply "*Option Choice" settings from the attribute value...
+	*/
+
+        if (ppd_mark_choices(ppd, attr->value))
+	  conflict = 1;
+      }
+    }
+    else if (!strcasecmp(optptr->name, "mirror"))
+    {
+      if (ppdMarkOption(ppd, "MirrorPrint", optptr->value))
+	conflict = 1;
+    }
     else if (ppdMarkOption(ppd, optptr->name, optptr->value))
       conflict = 1;
 
+  debug_marked(ppd, "After...");
+
   return (conflict);
 }
 
@@ -396,18 +470,28 @@ cupsParseOptions(
   char	*copyarg,			/* Copy of input string */
 	*ptr,				/* Pointer into string */
 	*name,				/* Pointer to name */
-	*value;				/* Pointer to value */
+	*value,				/* Pointer to value */
+	quote;				/* Quote character */
 
 
-  if (arg == NULL || options == NULL || num_options < 0)
+ /*
+  * Range check input...
+  */
+
+  if (!arg)
+    return (num_options);
+
+  if (!options || num_options < 0)
     return (0);
 
  /*
   * Make a copy of the argument string and then divide it up...
   */
 
-  copyarg     = strdup(arg);
-  ptr         = copyarg;
+  if ((copyarg = strdup(arg)) == NULL)
+    return (num_options);
+
+  ptr = copyarg;
 
  /*
   * Skip leading spaces...
@@ -427,7 +511,7 @@ cupsParseOptions(
     */
 
     name = ptr;
-    while (!isspace(*ptr & 255) && *ptr != '=' && *ptr != '\0')
+    while (!isspace(*ptr & 255) && *ptr != '=' && *ptr)
       ptr ++;
 
    /*
@@ -447,10 +531,10 @@ cupsParseOptions(
     if (*ptr != '=')
     {
      /*
-      * Start of another option...
+      * Boolean option...
       */
 
-      if (strncasecmp(name, "no", 2) == 0)
+      if (!strncasecmp(name, "no", 2))
         num_options = cupsAddOption(name + 2, "false", num_options,
 	                            options);
       else
@@ -465,38 +549,18 @@ cupsParseOptions(
 
     *ptr++ = '\0';
 
-    if (*ptr == '\'')
+    if (*ptr == '\'' || *ptr == '\"')
     {
      /*
       * Quoted string constant...
       */
 
-      ptr ++;
+      quote = *ptr++;
       value = ptr;
 
-      while (*ptr != '\'' && *ptr != '\0')
+      while (*ptr != quote && *ptr)
       {
-        if (*ptr == '\\')
-	  _cups_strcpy(ptr, ptr + 1);
-
-        ptr ++;
-      }
-
-      if (*ptr != '\0')
-        *ptr++ = '\0';
-    }
-    else if (*ptr == '\"')
-    {
-     /*
-      * Double-quoted string constant...
-      */
-
-      ptr ++;
-      value = ptr;
-
-      while (*ptr != '\"' && *ptr != '\0')
-      {
-        if (*ptr == '\\')
+        if (*ptr == '\\' && ptr[1])
 	  _cups_strcpy(ptr, ptr + 1);
 
         ptr ++;
@@ -529,7 +593,7 @@ cupsParseOptions(
 	      break;
 	  }
         }
-        else if (*ptr == '\\')
+        else if (*ptr == '\\' && ptr[1])
 	  _cups_strcpy(ptr, ptr + 1);
 
       if (*ptr != '\0')
@@ -543,9 +607,9 @@ cupsParseOptions(
 
       value = ptr;
 
-      while (!isspace(*ptr & 255) && *ptr != '\0')
+      while (!isspace(*ptr & 255) && *ptr)
       {
-        if (*ptr == '\\')
+        if (*ptr == '\\' && ptr[1])
 	  _cups_strcpy(ptr, ptr + 1);
 
         ptr ++;
@@ -633,6 +697,110 @@ cupsRemoveOption(
 }
 
 
+#ifdef DEBUG
 /*
- * End of "$Id: options.c 6313 2007-02-27 15:46:10Z mike $".
+ * 'debug_marked()' - Output the marked array to stdout...
+ */
+
+static void
+debug_marked(ppd_file_t *ppd,		/* I - PPD file data */
+             const char *title)		/* I - Title for list */
+{
+  ppd_choice_t	*c;			/* Current choice */
+
+
+  printf("cupsMarkOptions: %s\n", title);
+
+  for (c = (ppd_choice_t *)cupsArrayFirst(ppd->marked);
+       c;
+       c = (ppd_choice_t *)cupsArrayNext(ppd->marked))
+    printf("cupsMarkOptions: %s=%s\n", c->option->keyword, c->choice);
+}
+#endif /* DEBUG */
+
+
+/*
+ * 'ppd_mark_choices()' - Mark one or more option choices from a string.
+ */
+
+static int				/* O - 1 if there are conflicts, 0 otherwise */
+ppd_mark_choices(ppd_file_t *ppd,	/* I - PPD file */
+                 const char *options)	/* I - "*Option Choice ..." string */
+{
+  char	option[PPD_MAX_NAME],		/* Current option */
+	choice[PPD_MAX_NAME],		/* Current choice */
+	*ptr;				/* Pointer into option or choice */
+  int	conflict = 0;			/* Do we have a conflict? */
+
+
+  if (!options)
+    return (0);
+
+ /*
+  * Read all of the "*Option Choice" pairs from the string, marking PPD
+  * options as we go...
+  */
+
+  while (*options)
+  {
+   /*
+    * Skip leading whitespace...
+    */
+
+    while (isspace(*options & 255))
+      options ++;
+
+    if (*options != '*')
+      break;
+
+   /*
+    * Get the option name...
+    */
+
+    options ++;
+    ptr = option;
+    while (*options && !isspace(*options & 255) &&
+	       ptr < (option + sizeof(option) - 1))
+      *ptr++ = *options++;
+
+    if (ptr == option)
+      break;
+
+    *ptr = '\0';
+
+   /*
+    * Get the choice...
+    */
+
+    while (isspace(*options & 255))
+      options ++;
+
+    if (!*options)
+      break;
+
+    ptr = choice;
+    while (*options && !isspace(*options & 255) &&
+	       ptr < (choice + sizeof(choice) - 1))
+      *ptr++ = *options++;
+
+    *ptr = '\0';
+
+   /*
+    * Mark the option...
+    */
+
+    if (ppdMarkOption(ppd, option, choice))
+      conflict = 1;
+  }
+
+ /*
+  * Return whether we had any conflicts...
+  */
+
+  return (conflict);
+}
+
+
+/*
+ * End of "$Id: options.c 7262 2008-01-28 23:10:10Z mike $".
  */
