@@ -1,33 +1,26 @@
 /*
- * "$Id: request.c 6419 2007-03-30 18:39:36Z mike $"
+ * "$Id: request.c 7296 2008-02-12 00:20:32Z mike $"
  *
  *   IPP utilities for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
  * Contents:
  *
  *   cupsDoFileRequest() - Do an IPP request with a file.
+ *   cupsDoIORequest()   - Do an IPP request with file descriptors.
  *   cupsDoRequest()     - Do an IPP request.
  *   _cupsSetError()     - Set the last IPP status code and status-message.
+ *   _cupsSetHTTPError() - Set the last error using the HTTP status.
  */
 
 /*
@@ -45,6 +38,9 @@
 #else
 #  include <unistd.h>
 #endif /* WIN32 || __EMX__ */
+#ifndef O_BINARY
+#  define O_BINARY 0
+#endif /* O_BINARY */
 
 
 /*
@@ -60,13 +56,67 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
                   ipp_t      *request,	/* I - IPP request */
                   const char *resource,	/* I - HTTP resource for POST */
 		  const char *filename)	/* I - File to send or NULL for none */
+{
+  ipp_t		*response;		/* IPP response data */
+  int		infile;			/* Input file */
+
+
+  if (filename)
+  {
+    if ((infile = open(filename, O_RDONLY | O_BINARY)) < 0)
+    {
+     /*
+      * Can't get file information!
+      */
+
+      _cupsSetError(errno == ENOENT ? IPP_NOT_FOUND : IPP_NOT_AUTHORIZED,
+                    strerror(errno));
+
+      ippDelete(request);
+
+      return (NULL);
+    }
+  }
+  else
+    infile = -1;
+
+  response = cupsDoIORequest(http, request, resource, infile, -1);
+
+  if (infile >= 0)
+    close(infile);
+
+  return (response);
+}
+
+
+/*
+ * 'cupsDoIORequest()' - Do an IPP request with file descriptors.
+ *
+ * This function sends the IPP request to the specified server, retrying
+ * and authenticating as necessary.  The request is freed with ippDelete()
+ * after receiving a valid IPP response.
+ *
+ * If "infile" is a valid file descriptor, cupsDoIORequest() copies
+ * all of the data from the file after the IPP request message.
+ *
+ * If "outfile" is a valid file descriptor, cupsDoIORequest() copies
+ * all of the data after the IPP response message to the file.
+ *
+ * @since CUPS 1.3@
+ */
+
+ipp_t *					/* O - Response data */
+cupsDoIORequest(http_t     *http,	/* I - HTTP connection to server */
+                ipp_t      *request,	/* I - IPP request */
+                const char *resource,	/* I - HTTP resource for POST */
+		int        infile,	/* I - File to read from or -1 for none */
+		int        outfile)	/* I - File to write to or -1 for none */
 {
   ipp_t		*response;		/* IPP response data */
   size_t	length;			/* Content-Length value */
   http_status_t	status;			/* Status of HTTP request */
   int		got_status;		/* Did we get the status? */
   ipp_state_t	state;			/* State of IPP processing */
-  FILE		*file;			/* File to send */
   struct stat	fileinfo;		/* File information */
   int		bytes;			/* Number of bytes read/written */
   char		buffer[32768];		/* Output buffer */
@@ -91,16 +141,16 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
   * See if we have a file to send...
   */
 
-  if (filename != NULL)
+  if (infile >= 0)
   {
-    if (stat(filename, &fileinfo))
+    if (fstat(infile, &fileinfo))
     {
      /*
       * Can't get file information!
       */
 
       _cupsSetError(errno == ENOENT ? IPP_NOT_FOUND : IPP_NOT_AUTHORIZED,
-                     strerror(errno));
+                    strerror(errno));
 
       ippDelete(request);
 
@@ -123,23 +173,20 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
 
       return (NULL);
     }
-
-    if ((file = fopen(filename, "rb")) == NULL)
-    {
-     /*
-      * Can't open file!
-      */
-
-      _cupsSetError(errno == ENOENT ? IPP_NOT_FOUND : IPP_NOT_AUTHORIZED,
-                     strerror(errno));
-
-      ippDelete(request);
-
-      return (NULL);
-    }
   }
-  else
-    file = NULL;
+
+#ifdef HAVE_SSL
+ /*
+  * See if we have an auth-info attribute and are communicating over
+  * a non-local link.  If so, encrypt the link so that we can pass
+  * the authentication information securely...
+  */
+
+  if (ippFindAttribute(request, "auth-info", IPP_TAG_TEXT) &&
+      !httpAddrLocalhost(http->hostaddr) && !http->tls &&
+      httpEncryption(http, HTTP_ENCRYPT_REQUIRED))
+    return (NULL);
+#endif /* HAVE_SSL */
 
  /*
   * Loop until we can send the request without authorization problems.
@@ -158,8 +205,15 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
     */
 
     length = ippLength(request);
-    if (filename)
+    if (infile >= 0)
+    {
+#ifndef WIN32
+      if (!S_ISREG(fileinfo.st_mode))
+        length = 0;			/* Chunk when piping */
+      else
+#endif /* !WIN32 */
       length += fileinfo.st_size;
+    }
 
     httpClearFields(http);
     httpSetLength(http, length);
@@ -219,7 +273,7 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
     else if (httpCheck(http))
       status = httpUpdate(http);
 
-    if (status == HTTP_CONTINUE && state == IPP_DATA && filename)
+    if (status == HTTP_CONTINUE && state == IPP_DATA && infile >= 0)
     {
       DEBUG_puts("cupsDoFileRequest: file write...");
 
@@ -227,9 +281,12 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
       * Send the file...
       */
 
-      rewind(file);
+#ifndef WIN32
+      if (S_ISREG(fileinfo.st_mode))
+#endif /* WIN32 */
+      lseek(infile, 0, SEEK_SET);
 
-      while ((bytes = (int)fread(buffer, 1, sizeof(buffer), file)) > 0)
+      while ((bytes = (int)read(infile, buffer, sizeof(buffer))) > 0)
       {
 	if (httpCheck(http))
 	{
@@ -361,22 +418,27 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
 
 	break;
       }
+      else if (outfile >= 0)
+      {
+       /*
+        * Write trailing data to file...
+	*/
+
+	while ((bytes = (int)httpRead2(http, buffer, sizeof(buffer))) > 0)
+	  if (write(outfile, buffer, bytes) < bytes)
+	    break;
+      }
+      else
+      {
+       /*
+        * Flush any remaining data...
+        */
+
+        httpFlush(http);
+      }
     }
   }
 
- /*
-  * Close the file if needed...
-  */
-
-  if (filename != NULL)
-    fclose(file);
-
- /*
-  * Flush any remaining data...
-  */
-
-  httpFlush(http);
-
  /*
   * Delete the original request and return the response...
   */
@@ -395,44 +457,7 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
 		       ippErrorString(response->request.status.status_code));
   }
   else if (status != HTTP_OK)
-  {
-    switch (status)
-    {
-      case HTTP_NOT_FOUND :
-          _cupsSetError(IPP_NOT_FOUND, httpStatus(status));
-	  break;
-
-      case HTTP_UNAUTHORIZED :
-          _cupsSetError(IPP_NOT_AUTHORIZED, httpStatus(status));
-	  break;
-
-      case HTTP_FORBIDDEN :
-          _cupsSetError(IPP_FORBIDDEN, httpStatus(status));
-	  break;
-
-      case HTTP_BAD_REQUEST :
-          _cupsSetError(IPP_BAD_REQUEST, httpStatus(status));
-	  break;
-
-      case HTTP_REQUEST_TOO_LARGE :
-          _cupsSetError(IPP_REQUEST_VALUE, httpStatus(status));
-	  break;
-
-      case HTTP_NOT_IMPLEMENTED :
-          _cupsSetError(IPP_OPERATION_NOT_SUPPORTED, httpStatus(status));
-	  break;
-
-      case HTTP_NOT_SUPPORTED :
-          _cupsSetError(IPP_VERSION_NOT_SUPPORTED, httpStatus(status));
-	  break;
-
-      default :
-	  DEBUG_printf(("HTTP error %d mapped to IPP_SERVICE_UNAVAILABLE!\n",
-                	status));
-	  _cupsSetError(IPP_SERVICE_UNAVAILABLE, httpStatus(status));
-	  break;
-    }
-  }
+    _cupsSetHTTPError(status);
 
   return (response);
 }
@@ -482,5 +507,51 @@ _cupsSetError(ipp_status_t status,	/* I - IPP status code */
 
 
 /*
- * End of "$Id: request.c 6419 2007-03-30 18:39:36Z mike $".
+ * '_cupsSetHTTPError()' - Set the last error using the HTTP status.
+ */
+
+void
+_cupsSetHTTPError(http_status_t status)	/* I - HTTP status code */
+{
+  switch (status)
+  {
+    case HTTP_NOT_FOUND :
+	_cupsSetError(IPP_NOT_FOUND, httpStatus(status));
+	break;
+
+    case HTTP_UNAUTHORIZED :
+	_cupsSetError(IPP_NOT_AUTHORIZED, httpStatus(status));
+	break;
+
+    case HTTP_FORBIDDEN :
+	_cupsSetError(IPP_FORBIDDEN, httpStatus(status));
+	break;
+
+    case HTTP_BAD_REQUEST :
+	_cupsSetError(IPP_BAD_REQUEST, httpStatus(status));
+	break;
+
+    case HTTP_REQUEST_TOO_LARGE :
+	_cupsSetError(IPP_REQUEST_VALUE, httpStatus(status));
+	break;
+
+    case HTTP_NOT_IMPLEMENTED :
+	_cupsSetError(IPP_OPERATION_NOT_SUPPORTED, httpStatus(status));
+	break;
+
+    case HTTP_NOT_SUPPORTED :
+	_cupsSetError(IPP_VERSION_NOT_SUPPORTED, httpStatus(status));
+	break;
+
+    default :
+	DEBUG_printf(("HTTP error %d mapped to IPP_SERVICE_UNAVAILABLE!\n",
+		      status));
+	_cupsSetError(IPP_SERVICE_UNAVAILABLE, httpStatus(status));
+	break;
+  }
+}
+
+
+/*
+ * End of "$Id: request.c 7296 2008-02-12 00:20:32Z mike $".
  */
