@@ -1,25 +1,16 @@
 /*
- * "$Id: string.c 6346 2007-03-17 18:07:17Z mike $"
+ * "$Id: string.c 7084 2007-11-30 03:37:11Z mike $"
  *
  *   String functions for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -622,7 +613,7 @@ _cups_strcasecmp(const char *s,	/* I - First string */
 #ifndef HAVE_STRNCASECMP
 int					/* O - Result of comparison (-1, 0, or 1) */
 _cups_strncasecmp(const char *s,	/* I - First string */
-                 vconst char *t,	/* I - Second string */
+                  const char *t,	/* I - Second string */
 		  size_t     n)		/* I - Maximum number of characters to compare */
 {
   while (*s != '\0' && *t != '\0' && n > 0)
@@ -743,5 +734,5 @@ compare_sp_items(_cups_sp_item_t *a,	/* I - First item */
 
 
 /*
- * End of "$Id: string.c 6346 2007-03-17 18:07:17Z mike $".
+ * End of "$Id: string.c 7084 2007-11-30 03:37:11Z mike $".
  */
