@@ -1,25 +1,16 @@
 /*
- * "$Id: testipp.c,v 1.7 2004/06/29 03:44:52 mike Exp $"
+ * "$Id: testipp.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   IPP test program for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2004 by Easy Software Products.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -36,7 +27,7 @@
 #include <stdlib.h>
 #include <cups/string.h>
 #include <errno.h>
-#include "ipp.h"
+#include "ipp-private.h"
 #ifdef WIN32
 #  include <io.h>
 #else
@@ -57,13 +48,16 @@ ipp_uchar_t	collection[] =			/* Collection buffer */
 		  0x01, 0x01,			/* IPP version */
 		  0x00, 0x02,			/* Print-Job operation */
 		  0x00, 0x00, 0x00, 0x01,	/* Request ID */
+
 		  IPP_TAG_OPERATION,
+
 		  IPP_TAG_CHARSET,
 		  0x00, 0x12,			/* Name length + name */
 		  'a','t','t','r','i','b','u','t','e','s','-',
 		  'c','h','a','r','s','e','t',
 		  0x00, 0x05,			/* Value length + value */
 		  'u','t','f','-','8',
+
 		  IPP_TAG_LANGUAGE,
 		  0x00, 0x1b,			/* Name length + name */
 		  'a','t','t','r','i','b','u','t','e','s','-',
@@ -71,6 +65,7 @@ ipp_uchar_t	collection[] =			/* Collection buffer */
 		  'g','u','a','g','e',
 		  0x00, 0x02,			/* Value length + value */
 		  'e','n',
+
 		  IPP_TAG_URI,
 		  0x00, 0x0b,			/* Name length + name */
 		  'p','r','i','n','t','e','r','-','u','r','i',
@@ -78,22 +73,58 @@ ipp_uchar_t	collection[] =			/* Collection buffer */
 		  'i','p','p',':','/','/','l','o','c','a','l',
 		  'h','o','s','t','/','p','r','i','n','t','e',
 		  'r','s','/','f','o','o',
+
 		  IPP_TAG_JOB,			/* job group tag */
+
 		  IPP_TAG_BEGIN_COLLECTION,	/* begCollection tag */
 		  0x00, 0x09,			/* Name length + name */
 		  'm', 'e', 'd', 'i', 'a', '-', 'c', 'o', 'l',
 		  0x00, 0x00,			/* No value */
-		  IPP_TAG_MEMBERNAME,		/* memberAttrName tag */
-		  0x00, 0x00,			/* No name */
-		  0x00, 0x0b,			/* Value length + value */
-		  'm', 'e', 'd', 'i', 'a', '-', 'c', 'o', 'l', 'o', 'r',
-		  IPP_TAG_KEYWORD,		/* keyword tag */
-		  0x00, 0x00,			/* No name */
-		  0x00, 0x04,			/* Value length + value */
-		  'b', 'l', 'u', 'e',
+		    IPP_TAG_MEMBERNAME,		/* memberAttrName tag */
+		    0x00, 0x00,			/* No name */
+		    0x00, 0x0b,			/* Value length + value */
+		    'm', 'e', 'd', 'i', 'a', '-', 'c', 'o', 'l', 'o', 'r',
+		    IPP_TAG_KEYWORD,		/* keyword tag */
+		    0x00, 0x00,			/* No name */
+		    0x00, 0x04,			/* Value length + value */
+		    'b', 'l', 'u', 'e',
+
+		    IPP_TAG_MEMBERNAME,		/* memberAttrName tag */
+		    0x00, 0x00,			/* No name */
+		    0x00, 0x0a,			/* Value length + value */
+		    'm', 'e', 'd', 'i', 'a', '-', 't', 'y', 'p', 'e',
+		    IPP_TAG_KEYWORD,		/* keyword tag */
+		    0x00, 0x00,			/* No name */
+		    0x00, 0x05,			/* Value length + value */
+		    'p', 'l', 'a', 'i', 'n',
 		  IPP_TAG_END_COLLECTION,	/* endCollection tag */
 		  0x00, 0x00,			/* No name */
 		  0x00, 0x00,			/* No value */
+
+		  IPP_TAG_BEGIN_COLLECTION,	/* begCollection tag */
+		  0x00, 0x00,			/* No name */
+		  0x00, 0x00,			/* No value */
+		    IPP_TAG_MEMBERNAME,		/* memberAttrName tag */
+		    0x00, 0x00,			/* No name */
+		    0x00, 0x0b,			/* Value length + value */
+		    'm', 'e', 'd', 'i', 'a', '-', 'c', 'o', 'l', 'o', 'r',
+		    IPP_TAG_KEYWORD,		/* keyword tag */
+		    0x00, 0x00,			/* No name */
+		    0x00, 0x05,			/* Value length + value */
+		    'p', 'l', 'a', 'i', 'd',
+
+		    IPP_TAG_MEMBERNAME,		/* memberAttrName tag */
+		    0x00, 0x00,			/* No name */
+		    0x00, 0x0a,			/* Value length + value */
+		    'm', 'e', 'd', 'i', 'a', '-', 't', 'y', 'p', 'e',
+		    IPP_TAG_KEYWORD,		/* keyword tag */
+		    0x00, 0x00,			/* No name */
+		    0x00, 0x06,			/* Value length + value */
+		    'g', 'l', 'o', 's', 's', 'y',
+		  IPP_TAG_END_COLLECTION,	/* endCollection tag */
+		  0x00, 0x00,			/* No name */
+		  0x00, 0x00,			/* No value */
+
 		  IPP_TAG_END			/* end tag */
 		};
 
@@ -102,10 +133,10 @@ ipp_uchar_t	collection[] =			/* Collection buffer */
  * Local functions...
  */
 
-void	hex_dump(ipp_uchar_t *buffer, int bytes);
+void	hex_dump(const char *title, ipp_uchar_t *buffer, int bytes);
 void	print_attributes(ipp_t *ipp, int indent);
-int	read_cb(void *data, ipp_uchar_t *buffer, int bytes);
-int	write_cb(void *data, ipp_uchar_t *buffer, int bytes);
+ssize_t	read_cb(void *data, ipp_uchar_t *buffer, size_t bytes);
+ssize_t	write_cb(void *data, ipp_uchar_t *buffer, size_t bytes);
 
 
 /*
@@ -116,99 +147,191 @@ int				/* O - Exit status */
 main(int  argc,			/* I - Number of command-line arguments */
      char *argv[])		/* I - Command-line arguments */
 {
-  ipp_t		*col;		/* Collection */
+  ipp_t		*cols[2];	/* Collections */
   ipp_t		*request;	/* Request */
   ipp_state_t	state;		/* State */
   int		length;		/* Length of data */
   int		fd;		/* File descriptor */
   int		i;		/* Looping var */
+  int		status;		/* Status of tests (0 = success, 1 = fail) */
 
 
-  request = ippNew();
-  request->request.op.version[0]   = 0x01;
-  request->request.op.version[1]   = 0x01;
-  request->request.op.operation_id = IPP_PRINT_JOB;
-  request->request.op.request_id   = 1;
+  status = 0;
 
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
-               "attributes-charset", NULL, "utf-8");
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
-               "attributes-natural-language", NULL, "en");
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
-               "printer-uri", NULL, "ipp://localhost/printers/foo");
-
-  col = ippNew();
-  ippAddString(col, IPP_TAG_JOB, IPP_TAG_KEYWORD, "media-color", NULL, "blue");
-  ippAddCollection(request, IPP_TAG_JOB, "media-col", col);
-
-  length = ippLength(request);
-  if (length != sizeof(collection))
-    printf("ERROR ippLength didn't compute the correct length (%d instead of %d bytes!)\n",
-           length, sizeof(collection));
-
-  wused = 0;
-  while ((state = ippWriteIO(wbuffer, write_cb, 1, NULL, request)) != IPP_DATA)
-    if (state == IPP_ERROR)
-      break;
-
-  if (state != IPP_DATA)
-    puts("ERROR writing collection attribute!");
-
-  printf("%d bytes written:\n", wused);
-  hex_dump(wbuffer, wused);
-
-  if (wused != sizeof(collection))
+  if (argc == 1)
   {
-    printf("ERROR expected %d bytes!\n", sizeof(collection));
-    hex_dump(collection, sizeof(collection));
-  }
-  else if (memcmp(wbuffer, collection, wused))
-  {
-    puts("ERROR output does not match baseline!");
-    hex_dump(collection, sizeof(collection));
-  }
+   /*
+    * Test request generation code...
+    */
 
-  ippDelete(col);
-  ippDelete(request);
-
-  request = ippNew();
-  rpos    = 0;
-
-  while ((state = ippReadIO(wbuffer, read_cb, 1, NULL, request)) != IPP_DATA)
-    if (state == IPP_ERROR)
-      break;
-
-  if (state != IPP_DATA)
-    puts("ERROR reading collection attribute!");
-
-  printf("%d bytes read.\n", rpos);
-
-  puts("Core IPP tests passed.");
-
-  for (i = 1; i < argc; i ++)
-  {
-    if ((fd = open(argv[i], O_RDONLY)) < 0)
-    {
-      printf("Unable to open \"%s\" - %s\n", argv[i], strerror(errno));
-      continue;
-    }
+    printf("Create Sample Request: ");
 
     request = ippNew();
-    while ((state = ippReadFile(fd, request)) == IPP_ATTRIBUTE);
+    request->request.op.version[0]   = 0x01;
+    request->request.op.version[1]   = 0x01;
+    request->request.op.operation_id = IPP_PRINT_JOB;
+    request->request.op.request_id   = 1;
+
+    ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
+        	 "attributes-charset", NULL, "utf-8");
+    ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
+        	 "attributes-natural-language", NULL, "en");
+    ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
+        	 "printer-uri", NULL, "ipp://localhost/printers/foo");
+
+    cols[0] = ippNew();
+    ippAddString(cols[0], IPP_TAG_JOB, IPP_TAG_KEYWORD, "media-color", NULL, "blue");
+    ippAddString(cols[0], IPP_TAG_JOB, IPP_TAG_KEYWORD, "media-type", NULL, "plain");
+
+    cols[1] = ippNew();
+    ippAddString(cols[1], IPP_TAG_JOB, IPP_TAG_KEYWORD, "media-color", NULL, "plaid");
+    ippAddString(cols[1], IPP_TAG_JOB, IPP_TAG_KEYWORD, "media-type", NULL, "glossy");
+
+    ippAddCollections(request, IPP_TAG_JOB, "media-col", 2, (const ipp_t **)cols);
+
+    length = ippLength(request);
+    if (length != sizeof(collection))
+    {
+      printf("FAIL - wrong ippLength(), %d instead of %d bytes!\n",
+             length, (int)sizeof(collection));
+      status = 1;
+    }
+    else
+      puts("PASS");
+
+   /*
+    * Write test #1...
+    */
+
+    printf("Write Sample to Memory: ");
+
+    wused = 0;
+    while ((state = ippWriteIO(wbuffer, write_cb, 1, NULL, request)) != IPP_DATA)
+      if (state == IPP_ERROR)
+	break;
 
     if (state != IPP_DATA)
-      printf("Error reading IPP message from \"%s\"!\n", argv[i]);
-    else
     {
-      printf("\n%s:\n", argv[i]);
-      print_attributes(request, 4);
+      printf("FAIL - %d bytes written.\n", wused);
+      status = 1;
     }
+    else if (wused != sizeof(collection))
+    {
+      printf("FAIL - wrote %d bytes, expected %d bytes!\n", wused,
+             (int)sizeof(collection));
+      hex_dump("Bytes Written", wbuffer, wused);
+      hex_dump("Baseline", collection, sizeof(collection));
+      status = 1;
+    }
+    else if (memcmp(wbuffer, collection, wused))
+    {
+      puts("FAIL - output does not match baseline!");
+      hex_dump("Bytes Written", wbuffer, wused);
+      hex_dump("Baseline", collection, sizeof(collection));
+      status = 1;
+    }
+    else
+      puts("PASS");
 
     ippDelete(request);
-    close(fd);
+
+   /*
+    * Read the data back in and confirm...
+    */
+
+    printf("Read Sample from Memory: ");
+
+    request = ippNew();
+    rpos    = 0;
+
+    while ((state = ippReadIO(wbuffer, read_cb, 1, NULL, request)) != IPP_DATA)
+      if (state == IPP_ERROR)
+	break;
+
+    length = ippLength(request);
+
+    if (state != IPP_DATA)
+    {
+      printf("FAIL - %d bytes read.\n", rpos);
+      status = 1;
+    }
+    else if (rpos != wused)
+    {
+      printf("FAIL - read %d bytes, expected %d bytes!\n", rpos, wused);
+      print_attributes(request, 8);
+      status = 1;
+    }
+    else if (length != sizeof(collection))
+    {
+      printf("FAIL - wrong ippLength(), %d instead of %d bytes!\n",
+             length, (int)sizeof(collection));
+      print_attributes(request, 8);
+      status = 1;
+    }
+    else
+      puts("PASS");
+
+    ippDelete(request);
+
+   /*
+    * Test _ippFindOption() private API...
+    */
+
+    fputs("_ippFindOption(\"printer-type\"): ", stdout);
+    if (_ippFindOption("printer-type"))
+      puts("PASS");
+    else
+    {
+      puts("FAIL");
+      status = 1;
+    }
+
+   /*
+    * Summarize...
+    */
+
+    putchar('\n');
+
+    if (status)
+      puts("Core IPP tests failed.");
+    else
+      puts("Core IPP tests passed.");
+  }
+  else
+  {
+   /*
+    * Read IPP files...
+    */
+
+    for (i = 1; i < argc; i ++)
+    {
+      if ((fd = open(argv[i], O_RDONLY)) < 0)
+      {
+	printf("Unable to open \"%s\" - %s\n", argv[i], strerror(errno));
+	status = 1;
+	continue;
+      }
+
+      request = ippNew();
+      while ((state = ippReadFile(fd, request)) == IPP_ATTRIBUTE);
+
+      if (state != IPP_DATA)
+      {
+	printf("Error reading IPP message from \"%s\"!\n", argv[i]);
+	status = 1;
+      }
+      else
+      {
+	printf("\n%s:\n", argv[i]);
+	print_attributes(request, 4);
+      }
+
+      ippDelete(request);
+      close(fd);
+    }
   }
 
-  return (0);
+  return (status);
 }
 
 
@@ -217,7 +340,8 @@ main(int  argc,			/* I - Number of command-line arguments */
  */
 
 void
-hex_dump(ipp_uchar_t *buffer,		/* I - Buffer to dump */
+hex_dump(const char  *title,		/* I - Title */
+         ipp_uchar_t *buffer,		/* I - Buffer to dump */
          int         bytes)		/* I - Number of bytes */
 {
   int	i, j;				/* Looping vars */
@@ -228,13 +352,15 @@ hex_dump(ipp_uchar_t *buffer,		/* I - Buffer to dump */
   * Show lines of 16 bytes at a time...
   */
 
+  printf("    %s:\n", title);
+
   for (i = 0; i < bytes; i += 16)
   {
    /*
     * Show the offset...
     */
 
-    printf("%04x ", i);
+    printf("    %04x ", i);
 
    /*
     * Then up to 16 bytes in hex...
@@ -362,7 +488,7 @@ print_attributes(ipp_t *ipp,		/* I - IPP request */
 
   for (group = IPP_TAG_ZERO, attr = ipp->attrs; attr; attr = attr->next)
   {
-    if (attr->group_tag == IPP_TAG_ZERO || !attr->name)
+    if ((attr->group_tag == IPP_TAG_ZERO && indent <= 8) || !attr->name)
     {
       group = IPP_TAG_ZERO;
       putchar('\n');
@@ -374,7 +500,7 @@ print_attributes(ipp_t *ipp,		/* I - IPP request */
       group = attr->group_tag;
 
       putchar('\n');
-      for (i = 2; i < indent; i ++)
+      for (i = 4; i < indent; i ++)
         putchar(' ');
 
       printf("%s:\n\n", tags[group]);
@@ -383,7 +509,10 @@ print_attributes(ipp_t *ipp,		/* I - IPP request */
     for (i = 0; i < indent; i ++)
       putchar(' ');
 
-    printf("%s (%s):", attr->name, tags[attr->value_tag]);
+    printf("%s (", attr->name);
+    if (attr->num_values > 1)
+      printf("1setOf ");
+    printf("%s):", tags[attr->value_tag]);
 
     switch (attr->value_tag)
     {
@@ -450,14 +579,17 @@ print_attributes(ipp_t *ipp,		/* I - IPP request */
           putchar('\n');
 
           for (i = 0, val = attr->values; i < attr->num_values; i ++, val ++)
+	  {
+	    if (i)
+	      putchar('\n');
 	    print_attributes(val->collection, indent + 4);
+	  }
           break;
 
       default :
-          putchar('\n');
+          printf("UNKNOWN (%d values)\n", attr->num_values);
           break;
     }
-
   }
 }
 
@@ -466,10 +598,10 @@ print_attributes(ipp_t *ipp,		/* I - IPP request */
  * 'read_cb()' - Read data from a buffer.
  */
 
-int					/* O - Number of bytes read */
+ssize_t					/* O - Number of bytes read */
 read_cb(void        *data,		/* I - Data */
         ipp_uchar_t *buffer,		/* O - Buffer to read */
-	int         bytes)		/* I - Number of bytes to read */
+	size_t      bytes)		/* I - Number of bytes to read */
 {
   int	count;				/* Number of bytes */
 
@@ -493,10 +625,10 @@ read_cb(void        *data,		/* I - Data */
  * 'write_cb()' - Write data into a buffer.
  */
 
-int					/* O - Number of bytes written */
+ssize_t					/* O - Number of bytes written */
 write_cb(void        *data,		/* I - Data */
          ipp_uchar_t *buffer,		/* I - Buffer to write */
-	 int         bytes)		/* I - Number of bytes to write */
+	 size_t      bytes)		/* I - Number of bytes to write */
 {
   int	count;				/* Number of bytes */
 
@@ -517,5 +649,5 @@ write_cb(void        *data,		/* I - Data */
 
 
 /*
- * End of "$Id: testipp.c,v 1.7 2004/06/29 03:44:52 mike Exp $".
+ * End of "$Id: testipp.c 6649 2007-07-11 21:46:42Z mike $".
  */
