@@ -1,25 +1,18 @@
 /*
- * "$Id: transcode.c 6188 2007-01-10 16:23:06Z mike $"
+ * "$Id: transcode.c 7224 2008-01-16 23:42:35Z mike $"
  *
  *   Transcoding support for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
- *   These coded instructions, statements, and computer programs are
- *   the property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the
- *   file "LICENSE.txt" which should have been included with this file.
- *   If this file is missing or damaged please contact Easy Software
- *   Products at:
+ *   These coded instructions, statements, and computer programs are the
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   This file is subject to the Apple OS-Developed Software exception.
  *
  * Contents:
  *
@@ -139,8 +132,6 @@ _cupsCharmapFlush(void)
     vnext = vmap->next;
 
     free_vbcs_charmap(vmap);
-
-    free(vmap);
   }
 
   vmap_cache = NULL;
@@ -337,13 +328,8 @@ cupsCharsetToUTF8(
 
   if (encoding < CUPS_ENCODING_SBCS_END)
     bytes = conv_sbcs_to_utf8(dest, (cups_sbcs_t *)src, maxout, encoding);
-  else if (encoding < CUPS_ENCODING_VBCS_END)
-    bytes = conv_vbcs_to_utf8(dest, (cups_sbcs_t *)src, maxout, encoding);
   else
-  {
-    DEBUG_puts("    Bad encoding, returning -1");
-    bytes = -1;
-  }
+    bytes = conv_vbcs_to_utf8(dest, (cups_sbcs_t *)src, maxout, encoding);
 
 #ifdef HAVE_PTHREAD_H
   pthread_mutex_unlock(&map_mutex);
@@ -444,10 +430,8 @@ cupsUTF8ToCharset(
 
   if (encoding < CUPS_ENCODING_SBCS_END)
     bytes = conv_utf8_to_sbcs((cups_sbcs_t *)dest, src, maxout, encoding);
-  else if (encoding < CUPS_ENCODING_VBCS_END)
-    bytes = conv_utf8_to_vbcs((cups_sbcs_t *)dest, src, maxout, encoding);
   else
-    bytes = -1;
+    bytes = conv_utf8_to_vbcs((cups_sbcs_t *)dest, src, maxout, encoding);
 
 #ifdef HAVE_PTHREAD_H
   pthread_mutex_unlock(&map_mutex);
@@ -1435,7 +1419,7 @@ get_vbcs_charmap(
   char		*s;			/* Line parsing pointer */
   char		line[256];		/* Line from charset map file */
   int		i;			/* Loop variable */
-  int		wide;			/* 32-bit legacy char */
+  int		legacy;			/* 32-bit legacy char */
 
 
   DEBUG_printf(("get_vbcs_charmap(encoding=%d, filename=\"%s\")\n",
@@ -1475,6 +1459,8 @@ get_vbcs_charmap(
   {
     DEBUG_puts("    Unable to get charmap count!");
 
+    cupsFileClose(fp);
+
     return (NULL);
   }
 
@@ -1486,9 +1472,10 @@ get_vbcs_charmap(
 
   if ((vmap = (_cups_vmap_t *)calloc(1, sizeof(_cups_vmap_t))) == NULL)
   {
-    cupsFileClose(fp);
     DEBUG_puts("    Unable to allocate memory!");
 
+    cupsFileClose(fp);
+
     return (NULL);
   }
 
@@ -1504,8 +1491,8 @@ get_vbcs_charmap(
 
   cupsFileRewind(fp);
 
-  i    = 0;
-  wide = 0;
+  i      = 0;
+  legacy = 0;
 
   while (cupsFileGets(fp, line, sizeof(line)))
   {
@@ -1575,9 +1562,9 @@ get_vbcs_charmap(
       * Save VBCS 32-bit to Unicode mapping in sorted list table...
       */
 
-      if (!wide)
+      if (!legacy)
       {
-	wide            = 1;
+	legacy          = 1;
 	vmap->widecount = (mapcount - i + 1);
 	wide2uni        = (_cups_wide2uni_t *)calloc(vmap->widecount,
 	                                             sizeof(_cups_wide2uni_t));
@@ -1655,5 +1642,5 @@ get_vbcs_charmap(
 
 
 /*
- * End of "$Id: transcode.c 6188 2007-01-10 16:23:06Z mike $"
+ * End of "$Id: transcode.c 7224 2008-01-16 23:42:35Z mike $"
  */
