@@ -1,25 +1,16 @@
 /*
- * "$Id: util.c 6145 2006-12-06 20:10:16Z mike $"
+ * "$Id: util.c 7224 2008-01-16 23:42:35Z mike $"
  *
  *   Printing utilities for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -40,6 +31,7 @@
  *   cupsGetPPD2()          - Get the PPD file for a printer on the specified
  *                            server.
  *   cupsGetPrinters()      - Get a list of printers from the default server.
+ *   cupsGetServerPPD()     - Get an available PPD file from the server.
  *   cupsLastError()        - Return the last IPP status code.
  *   cupsLastErrorString()  - Return the last IPP status-message.
  *   cupsPrintFile()        - Print a file to a printer or class on the default
@@ -99,7 +91,6 @@ cupsCancelJob(const char *name,		/* I - Name of printer or class */
 		uri[HTTP_MAX_URI];	/* Printer URI */
   ipp_t		*request,		/* IPP request */
 		*response;		/* IPP response */
-  cups_lang_t	*language;		/* Language info */
   _cups_globals_t *cg = _cupsGlobals();	/* Pointer to library globals */
 
 
@@ -137,21 +128,7 @@ cupsCancelJob(const char *name,		/* I - Name of printer or class */
   *    [requesting-user-name]
   */
 
-  request = ippNew();
-
-  request->request.op.operation_id = IPP_CANCEL_JOB;
-  request->request.op.request_id   = 1;
-
-  language = cupsLangDefault();
-
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
-               "attributes-charset", NULL, cupsLangEncoding(language));
-
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
-               "attributes-natural-language", NULL,
-               language != NULL ? language->language : "C");
-
-  cupsLangFree(language);
+  request = ippNewRequest(IPP_CANCEL_JOB);
 
   ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri",
                NULL, uri);
@@ -213,7 +190,6 @@ cupsGetClasses(char ***classes)		/* O - Classes */
   ipp_t		*request,		/* IPP Request */
 		*response;		/* IPP Response */
   ipp_attribute_t *attr;		/* Current attribute */
-  cups_lang_t	*language;		/* Default language */
   char		**temp;			/* Temporary pointer */
   _cups_globals_t *cg = _cupsGlobals();	/* Pointer to library globals */
 
@@ -245,20 +221,7 @@ cupsGetClasses(char ***classes)		/* O - Classes */
   *    requested-attributes
   */
 
-  request = ippNew();
-
-  request->request.op.operation_id = CUPS_GET_CLASSES;
-  request->request.op.request_id   = 1;
-
-  language = cupsLangDefault();
-
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
-               "attributes-charset", NULL, cupsLangEncoding(language));
-
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
-               "attributes-natural-language", NULL, language->language);
-
-  cupsLangFree(language);
+  request = ippNewRequest(CUPS_GET_CLASSES);
 
   ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
                "requested-attributes", NULL, "printer-name");
@@ -379,7 +342,6 @@ cupsGetDefault2(http_t *http)		/* I - HTTP connection */
   ipp_t		*request,		/* IPP Request */
 		*response;		/* IPP Response */
   ipp_attribute_t *attr;		/* Current attribute */
-  cups_lang_t	*language;		/* Default language */
   const char	*var;			/* Environment variable */
   _cups_globals_t *cg = _cupsGlobals();	/* Pointer to library globals */
 
@@ -411,20 +373,7 @@ cupsGetDefault2(http_t *http)		/* I - HTTP connection */
   *    attributes-natural-language
   */
 
-  request = ippNew();
-
-  request->request.op.operation_id = CUPS_GET_DEFAULT;
-  request->request.op.request_id   = 1;
-
-  language = cupsLangDefault();
-
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
-               "attributes-charset", NULL, cupsLangEncoding(language));
-
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
-               "attributes-natural-language", NULL, language->language);
-
-  cupsLangFree(language);
+  request = ippNewRequest(CUPS_GET_DEFAULT);
 
  /*
   * Do the request and get back a response...
@@ -499,7 +448,6 @@ cupsGetJobs2(http_t     *http,		/* I - HTTP connection */
   ipp_t		*request,		/* IPP Request */
 		*response;		/* IPP Response */
   ipp_attribute_t *attr;		/* Current attribute */
-  cups_lang_t	*language;		/* Default language */
   cups_job_t	*temp;			/* Temporary pointer */
   int		id,			/* job-id */
 		priority,		/* job-priority */
@@ -572,20 +520,7 @@ cupsGetJobs2(http_t     *http,		/* I - HTTP connection */
   *    requested-attributes
   */
 
-  request = ippNew();
-
-  request->request.op.operation_id = IPP_GET_JOBS;
-  request->request.op.request_id   = 1;
-
-  language = cupsLangDefault();
-
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
-               "attributes-charset", NULL, cupsLangEncoding(language));
-
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
-               "attributes-natural-language", NULL, language->language);
-
-  cupsLangFree(language);
+  request = ippNewRequest(IPP_GET_JOBS);
 
   ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
                "printer-uri", NULL, uri);
@@ -911,9 +846,6 @@ cupsGetPPD2(http_t     *http,		/* I - HTTP connection */
 
   close(fd);
 
-  if (http2 != http)
-    httpClose(http2);
-
  /*
   * See if we actually got the file or an error...
   */
@@ -942,6 +874,9 @@ cupsGetPPD2(http_t     *http,		/* I - HTTP connection */
     return (NULL);
   }
 
+  if (http2 != http)
+    httpClose(http2);
+
  /*
   * Return the PPD file...
   */
@@ -965,7 +900,6 @@ cupsGetPrinters(char ***printers)	/* O - Printers */
   ipp_t		*request,		/* IPP Request */
 		*response;		/* IPP Response */
   ipp_attribute_t *attr;		/* Current attribute */
-  cups_lang_t	*language;		/* Default language */
   char		**temp;			/* Temporary pointer */
   _cups_globals_t *cg = _cupsGlobals();	/* Pointer to library globals */
 
@@ -997,20 +931,7 @@ cupsGetPrinters(char ***printers)	/* O - Printers */
   *    requested-attributes
   */
 
-  request = ippNew();
-
-  request->request.op.operation_id = CUPS_GET_PRINTERS;
-  request->request.op.request_id   = 1;
-
-  language = cupsLangDefault();
-
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
-               "attributes-charset", NULL, cupsLangEncoding(language));
-
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
-               "attributes-natural-language", NULL, language->language);
-
-  cupsLangFree(language);
+  request = ippNewRequest(CUPS_GET_PRINTERS);
 
   ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
                "requested-attributes", NULL, "printer-name");
@@ -1069,6 +990,82 @@ cupsGetPrinters(char ***printers)	/* O - Printers */
 }
 
 
+/*
+ * 'cupsGetServerPPD()' - Get an available PPD file from the server.
+ *
+ * This function returns the named PPD file from the server.  The
+ * list of available PPDs is provided by the IPP CUPS_GET_PPDS
+ * operation.
+ *
+ * You must remove (unlink) the PPD file when you are finished with
+ * it. The PPD filename is stored in a static location that will be
+ * overwritten on the next call to cupsGetPPD(), cupsGetPPD2(), or
+ * cupsGetServerPPD().
+ *
+ * @since CUPS 1.3@
+ */
+
+char *					/* O - Name of PPD file or NULL on error */
+cupsGetServerPPD(http_t     *http,	/* I - HTTP connection */
+                 const char *name)	/* I - Name of PPD file ("ppd-name") */
+{
+  int			fd;		/* PPD file descriptor */
+  ipp_t			*request;	/* IPP request */
+  _cups_globals_t	*cg = _cupsGlobals();
+					/* Pointer to library globals */
+
+
+ /*
+  * Range check input...
+  */
+
+  if (!http || !name)
+  {
+    if (!http)
+      _cupsSetError(IPP_INTERNAL_ERROR, "No HTTP connection!");
+    else
+      _cupsSetError(IPP_INTERNAL_ERROR, "No PPD name!");
+
+    return (NULL);
+  }
+
+ /*
+  * Get a temp file...
+  */
+
+  if ((fd = cupsTempFd(cg->ppd_filename, sizeof(cg->ppd_filename))) < 0)
+  {
+   /*
+    * Can't open file; close the server connection and return NULL...
+    */
+
+    _cupsSetError(IPP_INTERNAL_ERROR, strerror(errno));
+
+    return (NULL);
+  }
+
+ /*
+  * Get the PPD file...
+  */
+
+  request = ippNewRequest(CUPS_GET_PPD);
+  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "ppd-name", NULL,
+               name);
+
+  ippDelete(cupsDoIORequest(http, request, "/", -1, fd));
+
+  close(fd);
+
+  if (cupsLastError() != IPP_OK)
+  {
+    unlink(cg->ppd_filename);
+    return (NULL);
+  }
+  else
+    return (cg->ppd_filename);
+}
+
+
 /*
  * 'cupsLastError()' - Return the last IPP status code.
  */
@@ -1202,7 +1199,6 @@ cupsPrintFiles2(http_t        *http,	/* I - HTTP connection */
   ipp_t		*response;		/* IPP response */
   ipp_attribute_t *attr;		/* IPP job-id attribute */
   char		uri[HTTP_MAX_URI];	/* Printer URI */
-  cups_lang_t	*language;		/* Language to use */
   int		jobid;			/* New job ID */
   const char	*base;			/* Basename of current filename */
 
@@ -1234,35 +1230,19 @@ cupsPrintFiles2(http_t        *http,	/* I - HTTP connection */
     return (0);
   }
 
- /*
-  * Setup the request data...
-  */
-
-  language = cupsLangDefault();
-
  /*
   * Build a standard CUPS URI for the printer and fill the standard IPP
   * attributes...
   */
 
-  if ((request = ippNew()) == NULL)
+  if ((request = ippNewRequest(num_files == 1 ? IPP_PRINT_JOB :
+                                                IPP_CREATE_JOB)) == NULL)
   {
     _cupsSetError(IPP_INTERNAL_ERROR, NULL);
 
     return (0);
   }
 
-  request->request.op.operation_id = num_files == 1 ? IPP_PRINT_JOB :
-                                                      IPP_CREATE_JOB;
-  request->request.op.request_id   = 1;
-
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
-               "attributes-charset", NULL, cupsLangEncoding(language));
-
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
-               "attributes-natural-language", NULL,
-               language != NULL ? language->language : "C");
-
   ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri",
                NULL, uri);
 
@@ -1324,21 +1304,11 @@ cupsPrintFiles2(http_t        *http,	/* I - HTTP connection */
       * attributes...
       */
 
-      if ((request = ippNew()) == NULL)
+      if ((request = ippNewRequest(IPP_SEND_DOCUMENT)) == NULL)
 	return (0);
 
-      request->request.op.operation_id = IPP_SEND_DOCUMENT;
-      request->request.op.request_id   = 1;
-
       snprintf(uri, sizeof(uri), "ipp://localhost/jobs/%d", jobid);
 
-      ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
-        	   "attributes-charset", NULL, cupsLangEncoding(language));
-
-      ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
-        	   "attributes-natural-language", NULL,
-        	   language != NULL ? language->language : "C");
-
       ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "job-uri",
         	   NULL, uri);
 
@@ -1390,8 +1360,6 @@ cupsPrintFiles2(http_t        *http,	/* I - HTTP connection */
 	ippDelete(response);
     }
 
-  cupsLangFree(language);
-
   return (jobid);
 }
 
@@ -1660,5 +1628,5 @@ cups_get_printer_uri(
 
 
 /*
- * End of "$Id: util.c 6145 2006-12-06 20:10:16Z mike $".
+ * End of "$Id: util.c 7224 2008-01-16 23:42:35Z mike $".
  */
