@@ -1,25 +1,16 @@
 /*
- * "$Id: hpgl-attr.c 4494 2005-02-18 02:18:11Z mike $"
+ * "$Id: hpgl-attr.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   HP-GL/2 attribute processing for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2005 by Easy Software Products.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1993-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -82,8 +73,9 @@ CR_color_range(int     num_params,	/* I - Number of parameters */
     ColorRange[2][1] = params[5].value.number - params[4].value.number;
   }
   else
-    fprintf(stderr, "WARNING: HP-GL/2 \'CR\' command with invalid number of parameters (%d)!\n",
-            num_params);
+    fprintf(stderr,
+            "DEBUG: HP-GL/2 \'CR\' command with invalid number of "
+	    "parameters (%d)!\n", num_params);
 }
 
 
@@ -208,8 +200,9 @@ NP_number_pens(int     num_params,	/* I - Number of parameters */
   else if (num_params == 1 && params[0].value.number <= 1024)
     PenCount = (int)params[0].value.number;
   else
-    fprintf(stderr, "WARNING: HP-GL/2 \'NP\' command with invalid number of parameters (%d)!\n",
-            num_params);
+    fprintf(stderr,
+            "DEBUG: HP-GL/2 \'NP\' command with invalid number of "
+	    "parameters (%d)!\n", num_params);
 
   for (i = 0; i <= PenCount; i ++)
     Pens[i].width = PenWidth;
@@ -292,8 +285,9 @@ PC_pen_color(int     num_params,	/* I - Number of parameters */
 	     Pens[PenNumber].width * PenScaling);
   }
   else
-    fprintf(stderr, "WARNING: HP-GL/2 \'PC\' command with invalid number of parameters (%d)!\n",
-            num_params);
+    fprintf(stderr,
+            "DEBUG: HP-GL/2 \'PC\' command with invalid number of "
+	    "parameters (%d)!\n", num_params);
 }
 
 
@@ -360,8 +354,9 @@ PW_pen_width(int     num_params,	/* I - Number of parameters */
 	     Pens[PenNumber].width * PenScaling);
   }
   else
-    fprintf(stderr, "WARNING: HP-GL/2 \'PW\' command with invalid number of parameters (%d)!\n",
-            num_params);
+    fprintf(stderr,
+            "DEBUG: HP-GL/2 \'PW\' command with invalid number of "
+	    "parameters (%d)!\n", num_params);
 }
 
 
@@ -408,8 +403,10 @@ SP_select_pen(int     num_params,	/* I - Number of parameters */
   else if (params[0].value.number <= PenCount)
     PenNumber = (int)params[0].value.number;
   else
-    fprintf(stderr, "WARNING: HP-GL/2 \'SP\' command with invalid number or value of parameters (%d, %d)!\n",
-            num_params, (int)params[0].value.number);
+    fprintf(stderr,
+            "DEBUG: HP-GL/2 \'SP\' command with invalid number or value "
+	    "of parameters (%d, %d)!\n", num_params,
+	    (int)params[0].value.number);
 
   if (PageDirty)
     printf("%.3f %.3f %.3f %.2f SP\n", Pens[PenNumber].rgb[0],
@@ -444,11 +441,12 @@ WU_width_units(int     num_params,	/* I - Number of parameters */
   else if (num_params == 1)
     WidthUnits = (int)params[0].value.number;
   else
-    fprintf(stderr, "WARNING: HP-GL/2 \'WU\' command with invalid number of parameters (%d)!\n",
-            num_params);
+    fprintf(stderr,
+            "DEBUG: HP-GL/2 \'WU\' command with invalid number of "
+	    "parameters (%d)!\n", num_params);
 }
 
 
 /*
- * End of "$Id: hpgl-attr.c 4494 2005-02-18 02:18:11Z mike $".
+ * End of "$Id: hpgl-attr.c 6649 2007-07-11 21:46:42Z mike $".
  */
