@@ -1,5 +1,5 @@
 /*
- * "$Id: image-gif.c 7220 2008-01-14 22:12:58Z mike $"
+ * "$Id: image-gif.c 7421 2008-04-01 20:32:12Z mike $"
  *
  *   GIF image routines for the Common UNIX Printing System (CUPS).
  *
@@ -37,6 +37,7 @@
 
 #define GIF_INTERLACE	0x40
 #define GIF_COLORMAP	0x80
+#define GIF_MAX_BITS	12
 
 typedef cups_ib_t	gif_cmap_t[256][4];
 typedef short		gif_table_t[4096];
@@ -462,7 +463,7 @@ gif_read_image(FILE         *fp,	/* I - Input file */
   pass      = 0;
   code_size = getc(fp);
 
-  if (!pixels)
+  if (code_size > GIF_MAX_BITS || !pixels)
     return (-1);
 
   if (gif_read_lzw(fp, 1, code_size) < 0)
@@ -692,5 +693,5 @@ gif_read_lzw(FILE *fp,			/* I - File to read from */
 
 
 /*
- * End of "$Id: image-gif.c 7220 2008-01-14 22:12:58Z mike $".
+ * End of "$Id: image-gif.c 7421 2008-04-01 20:32:12Z mike $".
  */
