@@ -1,25 +1,16 @@
 /*
- * "$Id: image.c 4741 2005-10-02 04:25:52Z mike $"
+ * "$Id: image.c 7222 2008-01-16 22:20:33Z mike $"
  *
  *   Base image support for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1993-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -329,15 +320,10 @@ cupsImageOpen(
   */
 
   if ((fp = fopen(filename, "r")) == NULL)
-  {
-/*    perror("ERROR: Unable to open image file");
-*/    return (NULL);
-  }
+    return (NULL);
 
   if (fread(header, 1, sizeof(header), fp) == 0)
   {
-/*    perror("ERROR: Unable to read image file header");
-*/
     fclose(fp);
     return (NULL);
   }
@@ -355,8 +341,7 @@ cupsImageOpen(
 
   if (img == NULL)
   {
-/*    perror("ERROR: Unable to allocate memory for image file");
-*/    fclose(fp);
+    fclose(fp);
     return (NULL);
   }
 
@@ -410,8 +395,7 @@ cupsImageOpen(
 #endif /* HAVE_LIBTIFF */
   else
   {
-/*    fputs("ERROR: Unknown image file format!");
-*/    fclose(fp);
+    fclose(fp);
     status = -1;
   }
 
@@ -738,8 +722,11 @@ get_tile(cups_image_t *img,		/* I - Image */
 
     DEBUG_printf(("Creating tile array (%dx%d)\n", xtiles, ytiles));
 
-    img->tiles = calloc(sizeof(cups_itile_t *), ytiles);
-    tile       = calloc(sizeof(cups_itile_t), xtiles * ytiles);
+    if ((img->tiles = calloc(sizeof(cups_itile_t *), ytiles)) == NULL)
+      return (NULL);
+
+    if ((tile = calloc(sizeof(cups_itile_t), xtiles * ytiles)) == NULL)
+      return (NULL);
 
     for (tiley = 0; tiley < ytiles; tiley ++)
     {
@@ -760,13 +747,23 @@ get_tile(cups_image_t *img,		/* I - Image */
   {
     if (img->num_ics < img->max_ics)
     {
-      ic         = calloc(sizeof(cups_ic_t) + bpp * CUPS_TILE_SIZE *
-                                              CUPS_TILE_SIZE, 1);
-      ic->pixels = ((cups_ib_t *)ic) + sizeof(cups_ic_t);
+      if ((ic = calloc(sizeof(cups_ic_t) +
+                       bpp * CUPS_TILE_SIZE * CUPS_TILE_SIZE, 1)) == NULL)
+      {
+        if (img->num_ics == 0)
+	  return (NULL);
 
-      img->num_ics ++;
+        flush_tile(img);
+	ic = img->first;
+      }
+      else
+      {
+	ic->pixels = ((cups_ib_t *)ic) + sizeof(cups_ic_t);
 
-      DEBUG_printf(("Allocated cache tile %d (%p)...\n", img->num_ics, ic));
+	img->num_ics ++;
+
+	DEBUG_printf(("Allocated cache tile %d (%p)...\n", img->num_ics, ic));
+      }
     }
     else
     {
@@ -841,5 +838,5 @@ get_tile(cups_image_t *img,		/* I - Image */
 
 
 /*
- * End of "$Id: image.c 4741 2005-10-02 04:25:52Z mike $".
+ * End of "$Id: image.c 7222 2008-01-16 22:20:33Z mike $".
  */
