@@ -1,5 +1,5 @@
 /*
- * "$Id: rastertohp.c 7222 2008-01-16 22:20:33Z mike $"
+ * "$Id: rastertohp.c 7406 2008-03-26 16:02:45Z mike $"
  *
  *   Hewlett-Packard Page Control Language filter for the Common UNIX
  *   Printing System (CUPS).
@@ -52,7 +52,8 @@ int		NumPlanes,		/* Number of color planes */
 		ColorBits,		/* Number of bits per color */
 		Feed,			/* Number of lines to skip */
 		Duplex,			/* Current duplex mode */
-		Page;			/* Current page number */
+		Page,			/* Current page number */
+		Canceled;		/* Has the current job been canceled? */
 
 
 /*
@@ -60,13 +61,13 @@ int		NumPlanes,		/* Number of color planes */
  */
 
 void	Setup(void);
-void	StartPage(ppd_file_t *ppd, cups_page_header_t *header);
+void	StartPage(ppd_file_t *ppd, cups_page_header2_t *header);
 void	EndPage(void);
 void	Shutdown(void);
 
 void	CancelJob(int sig);
 void	CompressData(unsigned char *line, int length, int plane, int type);
-void	OutputLine(cups_page_header_t *header);
+void	OutputLine(cups_page_header2_t *header);
 
 
 /*
@@ -91,31 +92,11 @@ Setup(void)
 
 void
 StartPage(ppd_file_t         *ppd,	/* I - PPD file */
-          cups_page_header_t *header)	/* I - Page header */
+          cups_page_header2_t *header)	/* I - Page header */
 {
   int	plane;				/* Looping var */
-#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
-  struct sigaction action;		/* Actions for POSIX signals */
-#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
- /*
-  * Register a signal handler to eject the current page if the
-  * job is cancelled.
-  */
-
-#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
-  sigset(SIGTERM, CancelJob);
-#elif defined(HAVE_SIGACTION)
-  memset(&action, 0, sizeof(action));
-
-  sigemptyset(&action.sa_mask);
-  action.sa_handler = CancelJob;
-  sigaction(SIGTERM, &action, NULL);
-#else
-  signal(SIGTERM, CancelJob);
-#endif /* HAVE_SIGSET */
-
  /*
   * Show page device dictionary...
   */
@@ -410,11 +391,6 @@ StartPage(ppd_file_t         *ppd,	/* I - PPD file */
 void
 EndPage(void)
 {
-#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
-  struct sigaction action;	/* Actions for POSIX signals */
-#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
-
-
  /*
   * Eject the current page...
   */
@@ -436,22 +412,6 @@ EndPage(void)
 
   fflush(stdout);
 
- /*
-  * Unregister the signal handler...
-  */
-
-#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
-  sigset(SIGTERM, SIG_IGN);
-#elif defined(HAVE_SIGACTION)
-  memset(&action, 0, sizeof(action));
-
-  sigemptyset(&action.sa_mask);
-  action.sa_handler = SIG_IGN;
-  sigaction(SIGTERM, &action, NULL);
-#else
-  signal(SIGTERM, SIG_IGN);
-#endif /* HAVE_SIGSET */
-
  /*
   * Free memory...
   */
@@ -489,26 +449,9 @@ Shutdown(void)
 void
 CancelJob(int sig)			/* I - Signal */
 {
-  int	i;				/* Looping var */
-
-
   (void)sig;
 
- /*
-  * Send out lots of NUL bytes to clear out any pending raster data...
-  */
-
-  for (i = 0; i < 600; i ++)
-    putchar(0);
-
- /*
-  * End the current page and exit...
-  */
-
-  EndPage();
-  Shutdown();
-
-  exit(0);
+  Canceled = 1;
 }
 
 
@@ -648,7 +591,7 @@ CompressData(unsigned char *line,	/* I - Data to compress */
  */
 
 void
-OutputLine(cups_page_header_t *header)	/* I - Page header */
+OutputLine(cups_page_header2_t *header)	/* I - Page header */
 {
   int		plane,			/* Current plane */
 		bytes,			/* Bytes to write */
@@ -731,15 +674,18 @@ OutputLine(cups_page_header_t *header)	/* I - Page header */
  * 'main()' - Main entry and processing of driver.
  */
 
-int			/* O - Exit status */
-main(int  argc,		/* I - Number of command-line arguments */
-     char *argv[])	/* I - Command-line arguments */
+int					/* O - Exit status */
+main(int  argc,				/* I - Number of command-line arguments */
+     char *argv[])			/* I - Command-line arguments */
 {
-  int			fd;	/* File descriptor */
-  cups_raster_t		*ras;	/* Raster stream for printing */
-  cups_page_header_t	header;	/* Page header from file */
-  int			y;	/* Current line */
-  ppd_file_t		*ppd;	/* PPD file */
+  int			fd;		/* File descriptor */
+  cups_raster_t		*ras;		/* Raster stream for printing */
+  cups_page_header2_t	header;		/* Page header from file */
+  int			y;		/* Current line */
+  ppd_file_t		*ppd;		/* PPD file */
+#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
+  struct sigaction action;		/* Actions for POSIX signals */
+#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
  /*
@@ -782,6 +728,25 @@ main(int  argc,		/* I - Number of command-line arguments */
 
   ras = cupsRasterOpen(fd, CUPS_RASTER_READ);
 
+ /*
+  * Register a signal handler to eject the current page if the
+  * job is cancelled.
+  */
+
+  Canceled = 0;
+
+#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
+  sigset(SIGTERM, CancelJob);
+#elif defined(HAVE_SIGACTION)
+  memset(&action, 0, sizeof(action));
+
+  sigemptyset(&action.sa_mask);
+  action.sa_handler = CancelJob;
+  sigaction(SIGTERM, &action, NULL);
+#else
+  signal(SIGTERM, CancelJob);
+#endif /* HAVE_SIGSET */
+
  /*
   * Initialize the print device...
   */
@@ -796,12 +761,15 @@ main(int  argc,		/* I - Number of command-line arguments */
 
   Page = 0;
 
-  while (cupsRasterReadHeader(ras, &header))
+  while (cupsRasterReadHeader2(ras, &header))
   {
    /*
     * Write a status message with the page number and number of copies.
     */
 
+    if (Canceled)
+      break;
+
     Page ++;
 
     fprintf(stderr, "PAGE: %d %d\n", Page, header.NumCopies);
@@ -822,6 +790,9 @@ main(int  argc,		/* I - Number of command-line arguments */
       * Let the user know how far we have progressed...
       */
 
+      if (Canceled)
+	break;
+
       if ((y & 127) == 0)
         fprintf(stderr, _("INFO: Printing page %d, %d%% complete...\n"), Page,
 	        100 * y / header.cupsHeight);
@@ -849,6 +820,9 @@ main(int  argc,		/* I - Number of command-line arguments */
     */
 
     EndPage();
+
+    if (Canceled)
+      break;
   }
 
  /*
@@ -882,5 +856,5 @@ main(int  argc,		/* I - Number of command-line arguments */
 
 
 /*
- * End of "$Id: rastertohp.c 7222 2008-01-16 22:20:33Z mike $".
+ * End of "$Id: rastertohp.c 7406 2008-03-26 16:02:45Z mike $".
  */
