@@ -1,25 +1,16 @@
 /*
- * "$Id: textcommon.c,v 1.32 2004/04/26 19:01:40 mike Exp $"
+ * "$Id: textcommon.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   Common text filter routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2004 by Easy Software Products.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
- *
- *       Voice: (301) 373-9603
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -35,6 +26,7 @@
  */
 
 #include "textcommon.h"
+#include <cups/i18n.h>
 
 
 /*
@@ -523,7 +515,7 @@ TextMain(const char *name,	/* I - Name of filter */
 
   if (argc < 6 || argc > 7)
   {
-    fprintf(stderr, "ERROR: %s job-id user title copies options [file]\n",
+    fprintf(stderr, _("Usage: %s job-id user title copies options [file]\n"),
             name);
     return (1);
   }
@@ -543,7 +535,7 @@ TextMain(const char *name,	/* I - Name of filter */
 
     if ((fp = fopen(argv[6], "rb")) == NULL)
     {
-      perror("ERROR: unable to open print file - ");
+      perror("DEBUG: unable to open print file - ");
       return (1);
     }
   }
@@ -568,9 +560,9 @@ TextMain(const char *name,	/* I - Name of filter */
 
     if ((val = getenv("CONTENT_TYPE")) == NULL)
     {
-      PrettyPrint = PRETTY_CODE;
-      NumKeywords = sizeof(code_keywords) / sizeof(code_keywords[0]);
-      Keywords    = code_keywords;
+      PrettyPrint = PRETTY_PLAIN;
+      NumKeywords = 0;
+      Keywords    = NULL;
     }
     else if (strcasecmp(val, "application/x-cshell") == 0)
     {
@@ -578,6 +570,12 @@ TextMain(const char *name,	/* I - Name of filter */
       NumKeywords = sizeof(csh_keywords) / sizeof(csh_keywords[0]);
       Keywords    = csh_keywords;
     }
+    else if (strcasecmp(val, "application/x-csource") == 0)
+    {
+      PrettyPrint = PRETTY_CODE;
+      NumKeywords = sizeof(code_keywords) / sizeof(code_keywords[0]);
+      Keywords    = code_keywords;
+    }
     else if (strcasecmp(val, "application/x-perl") == 0)
     {
       PrettyPrint = PRETTY_PERL;
@@ -592,9 +590,9 @@ TextMain(const char *name,	/* I - Name of filter */
     }
     else
     {
-      PrettyPrint = PRETTY_CODE;
-      NumKeywords = sizeof(code_keywords) / sizeof(code_keywords[0]);
-      Keywords    = code_keywords;
+      PrettyPrint = PRETTY_PLAIN;
+      NumKeywords = 0;
+      Keywords    = NULL;
     }
   }
 
@@ -707,6 +705,8 @@ TextMain(const char *name,	/* I - Name of filter */
           }
 
 	  keycol = column;
+
+          attr &= ~ATTR_BOLD;
           break;
 
       case 0x0d :		/* CR */
@@ -874,14 +874,14 @@ TextMain(const char *name,	/* I - Name of filter */
           if (ch < ' ')
             break;		/* Ignore other control chars */
 
-          if (PrettyPrint)
+          if (PrettyPrint > PRETTY_PLAIN)
 	  {
 	   /*
 	    * Do highlighting of C/C++ keywords, preprocessor commands,
 	    * and comments...
 	    */
 
-	    if ((ch == ' ' || ch == '\t') && (attr & ATTR_BOLD))
+	    if (ch == ' ' && (attr & ATTR_BOLD))
 	    {
 	     /*
 	      * Stop bolding preprocessor command...
@@ -934,7 +934,7 @@ TextMain(const char *name,	/* I - Name of filter */
 	      */
 
 	      cstring = -1;
-              attr    |= ATTR_BLUE;
+              attr    = ATTR_BLUE;
 	    }
             else if (ch == '*' && lastch == '/' && !cstring &&
 	             PrettyPrint != PRETTY_SHELL)
@@ -944,7 +944,7 @@ TextMain(const char *name,	/* I - Name of filter */
 	      */
 
 	      ccomment = 1;
-	      attr     |= ATTR_ITALIC | ATTR_GREEN;
+	      attr     = ATTR_ITALIC | ATTR_GREEN;
 	    }
 	    else if (ch == '/' && lastch == '/' && !cstring &&
 	             PrettyPrint == PRETTY_CODE)
@@ -953,7 +953,7 @@ TextMain(const char *name,	/* I - Name of filter */
 	      * Start a C++-style comment...
 	      */
 
-	      attr |= ATTR_ITALIC | ATTR_GREEN;
+	      attr = ATTR_ITALIC | ATTR_GREEN;
 	    }
 	    else if (ch == '#' && !cstring && PrettyPrint != PRETTY_CODE)
 	    {
@@ -961,7 +961,7 @@ TextMain(const char *name,	/* I - Name of filter */
 	      * Start a shell-style comment...
 	      */
 
-	      attr |= ATTR_ITALIC | ATTR_GREEN;
+	      attr = ATTR_ITALIC | ATTR_GREEN;
 	    }
 	    else if (ch == '#' && column == 0 && !ccomment && !cstring &&
 	             PrettyPrint == PRETTY_CODE)
@@ -970,7 +970,7 @@ TextMain(const char *name,	/* I - Name of filter */
 	      * Start a preprocessor command...
 	      */
 
-	      attr |= ATTR_BOLD | ATTR_RED;
+	      attr = ATTR_BOLD | ATTR_RED;
 	    }
           }
 
@@ -1183,5 +1183,5 @@ getutf8(FILE *fp)	/* I - File to read from */
 
 
 /*
- * End of "$Id: textcommon.c,v 1.32 2004/04/26 19:01:40 mike Exp $".
+ * End of "$Id: textcommon.c 6649 2007-07-11 21:46:42Z mike $".
  */
