@@ -1,25 +1,16 @@
 /*
- * "$Id: texttops.c 6289 2007-02-19 13:00:10Z mike $"
+ * "$Id: texttops.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   Text to PostScript filter for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1993-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -38,6 +29,7 @@
  */
 
 #include "textcommon.h"
+#include <cups/i18n.h>
 
 
 /*
@@ -238,7 +230,7 @@ WriteProlog(const char *title,		/* I - Title of job */
   }
   else
   {
-    fprintf(stderr, "ERROR: Unable to open \"%s\" - %s\n", filename,
+    fprintf(stderr, _("ERROR: Unable to open \"%s\" - %s\n"), filename,
             strerror(errno));
     exit(1);
   }
@@ -258,7 +250,7 @@ WriteProlog(const char *title,		/* I - Title of job */
       * Can't open charset file!
       */
 
-      fprintf(stderr, "ERROR: Unable to open %s: %s\n", filename,
+      fprintf(stderr, _("ERROR: Unable to open %s: %s\n"), filename,
               strerror(errno));
       exit(1);
     }
@@ -274,7 +266,7 @@ WriteProlog(const char *title,		/* I - Title of job */
       */
 
       fclose(fp);
-      fprintf(stderr, "ERROR: Bad/empty charset file %s\n", filename);
+      fprintf(stderr, _("ERROR: Bad charset file %s\n"), filename);
       exit(1);
     }
 
@@ -285,7 +277,7 @@ WriteProlog(const char *title,		/* I - Title of job */
       */
 
       fclose(fp);
-      fprintf(stderr, "ERROR: Bad charset file %s\n", filename);
+      fprintf(stderr, _("ERROR: Bad charset file %s\n"), filename);
       exit(1);
     }
 
@@ -346,7 +338,7 @@ WriteProlog(const char *title,		/* I - Title of job */
 	  * Can't have a font without all required values...
 	  */
 
-	  fprintf(stderr, "ERROR: bad font description line: %s\n", valptr);
+	  fprintf(stderr, _("ERROR: Bad font description line: %s\n"), valptr);
 	  fclose(fp);
 	  exit(1);
 	}
@@ -359,7 +351,7 @@ WriteProlog(const char *title,		/* I - Title of job */
 	  Directions[NumFonts] = -1;
 	else
 	{
-	  fprintf(stderr, "ERROR: Bad text direction %s\n", valptr);
+	  fprintf(stderr, _("ERROR: Bad text direction %s\n"), valptr);
 	  fclose(fp);
 	  exit(1);
 	}
@@ -382,7 +374,7 @@ WriteProlog(const char *title,		/* I - Title of job */
 	  * Can't have a font without all required values...
 	  */
 
-	  fprintf(stderr, "ERROR: bad font description line: %s\n", valptr);
+	  fprintf(stderr, _("ERROR: Bad font description line: %s\n"), valptr);
 	  fclose(fp);
 	  exit(1);
 	}
@@ -395,7 +387,7 @@ WriteProlog(const char *title,		/* I - Title of job */
           Widths[NumFonts] = 2;
 	else 
 	{
-	  fprintf(stderr, "ERROR: Bad text width %s\n", valptr);
+	  fprintf(stderr, _("ERROR: Bad text width %s\n"), valptr);
 	  fclose(fp);
 	  exit(1);
 	}
@@ -510,7 +502,7 @@ WriteProlog(const char *title,		/* I - Title of job */
 	  * Can't have a font without all required values...
 	  */
 
-	  fprintf(stderr, "ERROR: bad font description line: %s\n", valptr);
+	  fprintf(stderr, _("ERROR: Bad font description line: %s\n"), valptr);
 	  fclose(fp);
 	  exit(1);
 	}
@@ -523,7 +515,7 @@ WriteProlog(const char *title,		/* I - Title of job */
 	  Directions[NumFonts] = -1;
 	else
 	{
-	  fprintf(stderr, "ERROR: Bad text direction %s\n", valptr);
+	  fprintf(stderr, _("ERROR: Bad text direction %s\n"), valptr);
 	  fclose(fp);
 	  exit(1);
 	}
@@ -546,7 +538,7 @@ WriteProlog(const char *title,		/* I - Title of job */
 	  * Can't have a font without all required values...
 	  */
 
-	  fprintf(stderr, "ERROR: bad font description line: %s\n", valptr);
+	  fprintf(stderr, _("ERROR: Bad font description line: %s\n"), valptr);
 	  fclose(fp);
 	  exit(1);
 	}
@@ -559,7 +551,7 @@ WriteProlog(const char *title,		/* I - Title of job */
           Widths[NumFonts] = 2;
 	else 
 	{
-	  fprintf(stderr, "ERROR: Bad text width %s\n", valptr);
+	  fprintf(stderr, _("ERROR: Bad text width %s\n"), valptr);
 	  fclose(fp);
 	  exit(1);
 	}
@@ -615,7 +607,7 @@ WriteProlog(const char *title,		/* I - Title of job */
     }
     else
     {
-      fprintf(stderr, "ERROR: Bad charset type %s\n", lineptr);
+      fprintf(stderr, _("ERROR: Bad charset type %s\n"), lineptr);
       fclose(fp);
       exit(1);
     }
@@ -1307,5 +1299,5 @@ write_text(const char *s)	/* I - String to write */
 
 
 /*
- * End of "$Id: texttops.c 6289 2007-02-19 13:00:10Z mike $".
+ * End of "$Id: texttops.c 6649 2007-07-11 21:46:42Z mike $".
  */
