@@ -22,7 +22,7 @@
 //------------------------------------------------------------------------
 
 struct DictEntry {
-  const char *key;
+  char *key;
   Object val;
 };
 
@@ -43,18 +43,18 @@ public:
   int getLength() { return length; }
 
   // Add an entry.  NB: does not copy key.
-  void add(const char *key, Object *val);
+  void add(char *key, Object *val);
 
   // Check if dictionary is of specified type.
-  GBool is(const char *type);
+  GBool is(char *type);
 
   // Look up an entry and return the value.  Returns a null object
   // if <key> is not in the dictionary.
-  Object *lookup(const char *key, Object *obj);
-  Object *lookupNF(const char *key, Object *obj);
+  Object *lookup(char *key, Object *obj);
+  Object *lookupNF(char *key, Object *obj);
 
   // Iterative accessors.
-  const char *getKey(int i);
+  char *getKey(int i);
   Object *getVal(int i, Object *obj);
   Object *getValNF(int i, Object *obj);
 
@@ -71,7 +71,7 @@ private:
   int length;			// number of entries in dictionary
   int ref;			// reference count
 
-  DictEntry *find(const char *key);
+  DictEntry *find(char *key);
 };
 
 #endif
