@@ -1,5 +1,5 @@
 //
-// "$Id: pdftops.cxx 6649 2007-07-11 21:46:42Z mike $"
+// "$Id: pdftops.cxx 7163 2008-01-02 23:08:19Z mike $"
 //
 //   PDF to PostScript filter front-end for the Common UNIX Printing
 //   System (CUPS).
@@ -308,8 +308,7 @@ main(int  argc,				// I - Number of command-line args
     // write PostScript file
     psOut = new PSOutputDev(psFileName->getCString(), doc->getXRef(),
                             doc->getCatalog(), 1, doc->getNumPages(),
-			    psModePS, 0, 0, 0, 0, gFalse,
-			    cupsGetOption("page-ranges", num_options, options));
+			    psModePS, 0, 0, 0, 0, gFalse, NULL);
     if (psOut->isOk())
       doc->displayPages(psOut, 1, doc->getNumPages(), 72.0, 72.0, 0,
                         gTrue, gFalse, gFalse);
@@ -341,5 +340,5 @@ main(int  argc,				// I - Number of command-line args
 
 
 //
-// End of "$Id: pdftops.cxx 6649 2007-07-11 21:46:42Z mike $".
+// End of "$Id: pdftops.cxx 7163 2008-01-02 23:08:19Z mike $".
 //
