@@ -1,25 +1,16 @@
 /*
- * "$Id: classes.c 6426 2007-04-02 13:13:27Z mike $"
+ * "$Id: classes.c 7224 2008-01-16 23:42:35Z mike $"
  *
  *   Printer class routines for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -265,6 +256,9 @@ cupsdFindAvailablePrinter(
     return (NULL);
   }
 
+  if (c->num_printers == 0)
+    return (NULL);
+
  /*
   * Make sure that the last printer is also a valid index into the printer
   * array.  If not, reset the last printer to 0...
@@ -399,7 +393,7 @@ cupsdLoadAllClasses(void)
       {
         cupsdLogMessage(CUPSD_LOG_ERROR,
 	                "Syntax error on line %d of classes.conf.", linenum);
-        return;
+        break;
       }
     }
     else if (!strcasecmp(line, "</Class>"))
@@ -413,14 +407,21 @@ cupsdLoadAllClasses(void)
       {
         cupsdLogMessage(CUPSD_LOG_ERROR,
 	                "Syntax error on line %d of classes.conf.", linenum);
-        return;
+        break;
       }
     }
     else if (!p)
     {
       cupsdLogMessage(CUPSD_LOG_ERROR,
                       "Syntax error on line %d of classes.conf.", linenum);
-      return;
+      break;
+    }
+    else if (!strcasecmp(line, "AuthInfoRequired"))
+    {
+      if (!cupsdSetAuthInfoRequired(p, value, NULL))
+	cupsdLogMessage(CUPSD_LOG_ERROR,
+			"Bad AuthInfoRequired on line %d of classes.conf.",
+			linenum);
     }
     else if (!strcasecmp(line, "Info"))
     {
@@ -457,7 +458,7 @@ cupsdLoadAllClasses(void)
       {
 	cupsdLogMessage(CUPSD_LOG_ERROR,
 	                "Syntax error on line %d of classes.conf.", linenum);
-	return;
+	break;
       }
       else if ((temp = cupsdFindPrinter(value)) == NULL)
       {
@@ -503,7 +504,7 @@ cupsdLoadAllClasses(void)
 	cupsdLogMessage(CUPSD_LOG_ERROR,
 	                "Syntax error on line %d of classes.conf.",
 	                linenum);
-	return;
+	break;
       }
     }
     else if (!strcasecmp(line, "StateMessage"))
@@ -545,7 +546,7 @@ cupsdLoadAllClasses(void)
 	cupsdLogMessage(CUPSD_LOG_ERROR,
 	                "Syntax error on line %d of classes.conf.",
 	                linenum);
-	return;
+	break;
       }
     }
     else if (!strcasecmp(line, "Shared"))
@@ -567,9 +568,9 @@ cupsdLoadAllClasses(void)
       else
       {
 	cupsdLogMessage(CUPSD_LOG_ERROR,
-	                "Syntax error on line %d of printers.conf.",
+	                "Syntax error on line %d of classes.conf.",
 	                linenum);
-	return;
+	break;
       }
     }
     else if (!strcasecmp(line, "JobSheets"))
@@ -608,7 +609,7 @@ cupsdLoadAllClasses(void)
       {
 	cupsdLogMessage(CUPSD_LOG_ERROR,
 	                "Syntax error on line %d of classes.conf.", linenum);
-	return;
+	break;
       }
     }
     else if (!strcasecmp(line, "AllowUser"))
@@ -622,7 +623,7 @@ cupsdLoadAllClasses(void)
       {
 	cupsdLogMessage(CUPSD_LOG_ERROR,
 	                "Syntax error on line %d of classes.conf.", linenum);
-	return;
+	break;
       }
     }
     else if (!strcasecmp(line, "DenyUser"))
@@ -636,7 +637,7 @@ cupsdLoadAllClasses(void)
       {
 	cupsdLogMessage(CUPSD_LOG_ERROR,
 	                "Syntax error on line %d of classes.conf.", linenum);
-	return;
+	break;
       }
     }
     else if (!strcasecmp(line, "QuotaPeriod"))
@@ -647,7 +648,7 @@ cupsdLoadAllClasses(void)
       {
 	cupsdLogMessage(CUPSD_LOG_ERROR,
 	                "Syntax error on line %d of classes.conf.", linenum);
-	return;
+	break;
       }
     }
     else if (!strcasecmp(line, "PageLimit"))
@@ -658,7 +659,7 @@ cupsdLoadAllClasses(void)
       {
 	cupsdLogMessage(CUPSD_LOG_ERROR,
 	                "Syntax error on line %d of classes.conf.", linenum);
-	return;
+	break;
       }
     }
     else if (!strcasecmp(line, "KLimit"))
@@ -669,7 +670,7 @@ cupsdLoadAllClasses(void)
       {
 	cupsdLogMessage(CUPSD_LOG_ERROR,
 	                "Syntax error on line %d of classes.conf.", linenum);
-	return;
+	break;
       }
     }
     else if (!strcasecmp(line, "OpPolicy"))
@@ -693,7 +694,7 @@ cupsdLoadAllClasses(void)
       {
 	cupsdLogMessage(CUPSD_LOG_ERROR,
 	                "Syntax error on line %d of classes.conf.", linenum);
-	return;
+	break;
       }
     }
     else if (!strcasecmp(line, "ErrorPolicy"))
@@ -704,7 +705,7 @@ cupsdLoadAllClasses(void)
       {
 	cupsdLogMessage(CUPSD_LOG_ERROR,
 	                "Syntax error on line %d of classes.conf.", linenum);
-	return;
+	break;
       }
     }
     else
@@ -812,6 +813,14 @@ cupsdSaveAllClasses(void)
     else
       cupsFilePrintf(fp, "<Class %s>\n", pclass->name);
 
+    if (pclass->num_auth_info_required > 0)
+    {
+      cupsFilePrintf(fp, "AuthInfoRequired %s", pclass->auth_info_required[0]);
+      for (i = 1; i < pclass->num_auth_info_required; i ++)
+        cupsFilePrintf(fp, ",%s", pclass->auth_info_required[i]);
+      cupsFilePutChar(fp, '\n');
+    }
+
     if (pclass->info)
     {
       if ((ptr = strchr(pclass->info, '#')) != NULL)
@@ -930,5 +939,5 @@ cupsdUpdateImplicitClasses(void)
 
 
 /*
- * End of "$Id: classes.c 6426 2007-04-02 13:13:27Z mike $".
+ * End of "$Id: classes.c 7224 2008-01-16 23:42:35Z mike $".
  */
