@@ -1,26 +1,17 @@
 /*
- * "$Id: conf.h 5696 2006-06-26 18:34:20Z mike $"
+ * "$Id: conf.h 7319 2008-02-15 23:26:51Z mike $"
  *
  *   Configuration file definitions for the Common UNIX Printing System (CUPS)
  *   scheduler.
  *
- *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  */
 
 
@@ -30,19 +21,19 @@
 
 typedef enum
 {
-  CUPSD_LOG_ATTR = -3,		/* Used internally for attributes */
-  CUPSD_LOG_STATE,		/* Used internally for state-reasons */
-  CUPSD_LOG_PAGE,		/* Used internally for page logging */
+  CUPSD_LOG_ATTR = -3,			/* Used internally for attributes */
+  CUPSD_LOG_STATE,			/* Used internally for state-reasons */
+  CUPSD_LOG_PAGE,			/* Used internally for page logging */
   CUPSD_LOG_NONE,
-  CUPSD_LOG_EMERG,		/* Emergency issues */
-  CUPSD_LOG_ALERT,		/* Something bad happened that needs attention */
-  CUPSD_LOG_CRIT,		/* Critical error but server continues */
-  CUPSD_LOG_ERROR,		/* Error condition */
-  CUPSD_LOG_WARN,		/* Warning */
-  CUPSD_LOG_NOTICE,		/* Normal condition that needs logging */
-  CUPSD_LOG_INFO,		/* General information */
-  CUPSD_LOG_DEBUG,		/* General debugging */
-  CUPSD_LOG_DEBUG2		/* Detailed debugging */
+  CUPSD_LOG_EMERG,			/* Emergency issues */
+  CUPSD_LOG_ALERT,			/* Something bad happened that needs attention */
+  CUPSD_LOG_CRIT,			/* Critical error but server continues */
+  CUPSD_LOG_ERROR,			/* Error condition */
+  CUPSD_LOG_WARN,			/* Warning */
+  CUPSD_LOG_NOTICE,			/* Normal condition that needs logging */
+  CUPSD_LOG_INFO,			/* General information */
+  CUPSD_LOG_DEBUG,			/* General debugging */
+  CUPSD_LOG_DEBUG2			/* Detailed debugging */
 } cupsd_loglevel_t;
 
 
@@ -50,8 +41,8 @@ typedef enum
  * Printcap formats...
  */
 
-#define PRINTCAP_BSD	0	/* Berkeley LPD format */
-#define PRINTCAP_SOLARIS 1	/* Solaris lpsched format */
+#define PRINTCAP_BSD		0	/* Berkeley LPD format */
+#define PRINTCAP_SOLARIS	1	/* Solaris lpsched format */
 
 
 /*
@@ -77,9 +68,11 @@ VAR char		*ConfigurationFile	VALUE(NULL),
 VAR int			ServerNameIsIP		VALUE(0);
 VAR int			NumSystemGroups		VALUE(0);
 					/* Number of system group names */
-VAR char		*SystemGroups[MAX_SYSTEM_GROUPS];
+VAR char		*SystemGroups[MAX_SYSTEM_GROUPS]
+						VALUE({0});
 					/* System group names */
-VAR gid_t		SystemGroupIDs[MAX_SYSTEM_GROUPS];
+VAR gid_t		SystemGroupIDs[MAX_SYSTEM_GROUPS]
+						VALUE({0});
 					/* System group IDs */
 VAR char		*AccessLog		VALUE(NULL),
 					/* Access log filename */
@@ -97,6 +90,8 @@ VAR char		*AccessLog		VALUE(NULL),
 					/* Default charset */
 			*DefaultLocale		VALUE(NULL),
 					/* Default locale */
+			*ErrorPolicy		VALUE(NULL),
+					/* Default printer-error-policy */
 			*RIPCache		VALUE(NULL),
 					/* Amount of memory for RIPs */
 			*TempDir		VALUE(NULL),
@@ -111,6 +106,12 @@ VAR char		*AccessLog		VALUE(NULL),
 					/* Remote root user */
 			*Classification		VALUE(NULL);
 					/* Classification of system */
+#ifdef HAVE_GSSAPI
+VAR char		*GSSServiceName		VALUE(NULL);
+					/* GSS service name */
+VAR char		*Krb5Keytab		VALUE(NULL);
+					/* Kerberos Keytab */
+#endif /* HAVE_GSSAPI */
 VAR uid_t		User			VALUE(1);
 					/* User ID for server */
 VAR gid_t		Group			VALUE(0);
@@ -127,7 +128,7 @@ VAR int			ClassifyOverride	VALUE(0),
 					/* Maximum number of clients */
 			MaxClientsPerHost	VALUE(0),
 					/* Maximum number of clients per host */
-			MaxCopies		VALUE(100),
+			MaxCopies		VALUE(CUPS_DEFAULT_MAX_COPIES),
 					/* Maximum number of copies per job */
 			MaxLogSize		VALUE(1024 * 1024),
 					/* Maximum size of log files */
@@ -161,7 +162,8 @@ VAR int			ClassifyOverride	VALUE(0),
 					/* Timeout before reload from SIGHUP */
 			RootCertDuration	VALUE(300),
 					/* Root certificate update interval */
-			RunUser,	/* User to run as, used for files */
+			RunUser			VALUE(0),
+					/* User to run as, used for files */
 			PrintcapFormat		VALUE(PRINTCAP_BSD),
 					/* Format of printcap file? */
 			DefaultShared		VALUE(TRUE);
@@ -195,21 +197,41 @@ VAR char		*LaunchdConf		VALUE(NULL);
 					/* launchd(8) configuration file */
 #endif /* HAVE_LAUNCHD */
 
+#ifdef __APPLE__
+VAR int			AppleQuotas		VALUE(TRUE);
+					/* Use Apple PrintService Quotas instead of CUPS quotas */
+#endif  /* __APPLE__ */
+
+#ifdef HAVE_AUTHORIZATION_H
+VAR char		*SystemGroupAuthKey	VALUE(NULL);
+					/* System group auth key */
+#endif /* HAVE_AUTHORIZATION_H */
+
+
 /*
  * Prototypes...
  */
 
+extern int	cupsdCheckPermissions(const char *filename,
+		                      const char *suffix, int mode,
+	 			      int user, int group, int is_dir,
+				      int create_dir);
 extern char	*cupsdGetDateTime(time_t t);
-extern int	cupsdReadConfiguration(void);
-extern int	cupsdLogRequest(cupsd_client_t *con, http_status_t code);
+#ifdef HAVE_GSSAPI
+extern int	cupsdLogGSSMessage(int level, int major_status,
+		                   int minor_status,
+		                   const char *message, ...);
+#endif /* HAVE_GSSAPI */
 extern int	cupsdLogMessage(int level, const char *message, ...)
 #ifdef __GNUC__
 __attribute__ ((__format__ (__printf__, 2, 3)))
 #endif /* __GNUC__ */
 ;
 extern int	cupsdLogPage(cupsd_job_t *job, const char *page);
+extern int	cupsdLogRequest(cupsd_client_t *con, http_status_t code);
+extern int	cupsdReadConfiguration(void);
 
 
 /*
- * End of "$Id: conf.h 5696 2006-06-26 18:34:20Z mike $".
+ * End of "$Id: conf.h 7319 2008-02-15 23:26:51Z mike $".
  */
