@@ -1,9 +1,9 @@
 /*
- * "$Id: cups-deviced.c 7013 2007-10-10 21:25:29Z mike $"
+ * "$Id: cups-deviced.c 7222 2008-01-16 22:20:33Z mike $"
  *
  *   Device scanning mini-daemon for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -291,6 +291,8 @@ main(int  argc,				/* I - Number of command-line args */
 	  if (!dev)
 	  {
             cupsDirClose(dir);
+	    fclose(fp);
+            kill(pid, SIGTERM);
 	    return (1);
 	  }
 
@@ -559,5 +561,5 @@ sigalrm_handler(int sig)		/* I - Signal number */
 
 
 /*
- * End of "$Id: cups-deviced.c 7013 2007-10-10 21:25:29Z mike $".
+ * End of "$Id: cups-deviced.c 7222 2008-01-16 22:20:33Z mike $".
  */
