@@ -1,25 +1,16 @@
 /*
- * "$Id: cups-lpd.c 6333 2007-03-12 20:49:13Z mike $"
+ * "$Id: cups-lpd.c 7222 2008-01-16 22:20:33Z mike $"
  *
  *   Line Printer Daemon interface for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -450,14 +441,41 @@ get_printer(http_t        *http,	/* I - HTTP connection */
     *options = NULL;
 
  /*
-  * If the queue name contains a space, lookup the printer-name using
-  * the printer-info value...
+  * See if the name is a queue name optionally with an instance name.
   */
 
-  if (strchr(name, ' '))
+  strlcpy(dest, name, destsize);
+  if ((value = strchr(dest, '/')) != NULL)
+    *value = '\0';
+
+ /*
+  * Setup the Get-Printer-Attributes request...
+  */
+
+  request = ippNewRequest(IPP_GET_PRINTER_ATTRIBUTES);
+
+  httpAssembleURIf(HTTP_URI_CODING_ALL, uri, sizeof(uri), "ipp", NULL,
+		   "localhost", 0, "/printers/%s", dest);
+
+  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri",
+	       NULL, uri);
+
+  ippAddStrings(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
+		"requested-attributes",
+		(int)(sizeof(requested) / sizeof(requested[0])),
+		NULL, requested);
+
+ /*
+  * Do the request...
+  */
+
+  response = cupsDoRequest(http, request, "/");
+
+  if (!response || cupsLastError() > IPP_OK_CONFLICT)
   {
    /*
-    * Lookup the printer-info...
+    * If we can't find the printer by name, look up the printer-name
+    * using the printer-info values...
     */
 
     ipp_attribute_t	*accepting_attr,/* printer-is-accepting-jobs */
@@ -467,6 +485,8 @@ get_printer(http_t        *http,	/* I - HTTP connection */
 			*state_attr;	/* printer-state */
 
 
+    ippDelete(response);
+
    /*
     * Setup the CUPS-Get-Printers request...
     */
@@ -579,47 +599,6 @@ get_printer(http_t        *http,	/* I - HTTP connection */
   }
   else
   {
-   /*
-    * Otherwise treat it as a queue name optionally with an instance name.
-    */
-
-    strlcpy(dest, name, destsize);
-    if ((value = strchr(dest, '/')) != NULL)
-      *value = '\0';
-
-   /*
-    * Setup the Get-Printer-Attributes request...
-    */
-
-    request = ippNewRequest(IPP_GET_PRINTER_ATTRIBUTES);
-
-    httpAssembleURIf(HTTP_URI_CODING_ALL, uri, sizeof(uri), "ipp", NULL,
-                     "localhost", 0, "/printers/%s", dest);
-
-    ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri",
-        	 NULL, uri);
-
-    ippAddStrings(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
-                  "requested-attributes",
-		  (int)(sizeof(requested) / sizeof(requested[0])),
-                  NULL, requested);
-
-   /*
-    * Do the request...
-    */
-
-    response = cupsDoRequest(http, request, "/");
-
-    if (!response || cupsLastError() > IPP_OK_CONFLICT)
-    {
-      syslog(LOG_ERR, "Unable to check printer status - %s",
-             cupsLastErrorString());
-
-      ippDelete(response);
-
-      return (-1);
-    }
-
    /*
     * Get values from the response...
     */
@@ -627,9 +606,9 @@ get_printer(http_t        *http,	/* I - HTTP connection */
     if (accepting)
     {
       if ((attr = ippFindAttribute(response, "printer-is-accepting-jobs",
-                        	   IPP_TAG_BOOLEAN)) == NULL)
+				   IPP_TAG_BOOLEAN)) == NULL)
 	syslog(LOG_ERR, "No printer-is-accepting-jobs attribute found in "
-                	"response from server!");
+			"response from server!");
       else
 	*accepting = attr->values[0].boolean;
     }
@@ -637,10 +616,10 @@ get_printer(http_t        *http,	/* I - HTTP connection */
     if (shared)
     {
       if ((attr = ippFindAttribute(response, "printer-is-shared",
-                        	   IPP_TAG_BOOLEAN)) == NULL)
+				   IPP_TAG_BOOLEAN)) == NULL)
       {
 	syslog(LOG_ERR, "No printer-is-shared attribute found in "
-                	"response from server!");
+			"response from server!");
 	*shared = 1;
       }
       else
@@ -650,9 +629,9 @@ get_printer(http_t        *http,	/* I - HTTP connection */
     if (state)
     {
       if ((attr = ippFindAttribute(response, "printer-state",
-                        	   IPP_TAG_ENUM)) == NULL)
+				   IPP_TAG_ENUM)) == NULL)
 	syslog(LOG_ERR, "No printer-state attribute found in "
-                	"response from server!");
+			"response from server!");
       else
 	*state = (ipp_pstate_t)attr->values[0].integer;
     }
@@ -667,13 +646,12 @@ get_printer(http_t        *http,	/* I - HTTP connection */
 #ifdef HAVE_CFPRIV_H /* MacOS X */
   if (shared && *shared)
   {
-    CFURLRef		prefsurl;	/* */
-    CFDataRef		xmldata;	/* */
-    CFPropertyListRef	plist;		/* */
-    CFStringRef		queueid;	/* */
-    CFArrayRef		lprqarray;	/* */
-    CFBooleanRef	serverflag;	/* */
-    Boolean		prefsok;	/* */
+    CFURLRef		prefsurl;	/* URL for preferences file */
+    CFDataRef		xmldata;	/* XML data from preferences file */
+    CFPropertyListRef	plist;		/* Property list from XML data */
+    CFStringRef		queueid;	/* CFString of destination name */
+    CFArrayRef		lprqarray;	/* Array of shared "LPR" printers */
+    CFBooleanRef	serverflag;	/* State of the print service */
     static const char printerprefsfile[] =
         "/Library/Preferences/com.apple.printservice.plist";
 					/* Preferences file */
@@ -702,22 +680,24 @@ get_printer(http_t        *http,	/* I - HTTP connection */
 		     false);
       if (prefsurl)
       {
-        prefsok = CFURLCreateDataAndPropertiesFromResource(
-	              kCFAllocatorDefault, prefsurl, &xmldata, 
-		      NULL, NULL, NULL);
-        if (prefsok)
+        if (CFURLCreateDataAndPropertiesFromResource(kCFAllocatorDefault,
+	                                             prefsurl, &xmldata, NULL,
+						     NULL, NULL))
 	{
 	  plist = CFPropertyListCreateFromXMLData(kCFAllocatorDefault, xmldata, 
-						  kCFPropertyListImmutable, NULL);
+						  kCFPropertyListImmutable,
+						  NULL);
 	  if (plist)
 	  {
 	    serverflag = (CFBooleanRef)CFDictionaryGetValue(
-	                     (CFDictionaryRef)plist, CFSTR("serviceState"));
+	                                   (CFDictionaryRef)plist,
+					   CFSTR("serviceState"));
 
             if (serverflag && CFBooleanGetValue(serverflag))
 	    {
 	      lprqarray = (CFArrayRef)CFDictionaryGetValue(
-	                      (CFDictionaryRef)plist, CFSTR("lprSharedQueues"));
+	                                  (CFDictionaryRef)plist,
+					  CFSTR("lprSharedQueues"));
 
 	      if (lprqarray)
 	      {
@@ -727,23 +707,20 @@ get_printer(http_t        *http,	/* I - HTTP connection */
 
                 if (queueid)
 		{
-	          *shared = CFArrayContainsValue(
-		        	lprqarray,
-		        	CFRangeMake(0, CFArrayGetCount(lprqarray)),
-				queueid);
+	          *shared = CFArrayContainsValue(lprqarray,
+						 CFRangeMake(0,
+						     CFArrayGetCount(lprqarray)),
+						 queueid);
 
                   CFRelease(queueid);
 		}
-
-                CFRelease(lprqarray);
 	      }
 	    }
 
-            if (serverflag)
-	      CFRelease(serverflag);
-
 	    CFRelease(plist);
 	  }
+
+	  CFRelease(xmldata);
 	}
 
 	CFRelease(prefsurl);
@@ -754,7 +731,7 @@ get_printer(http_t        *http,	/* I - HTTP connection */
 	                "on queue: %s", name);
     }
   }
-#endif	/* HAVE_CFPRIV_H */
+#endif /* HAVE_CFPRIV_H */
 
  /*
   * Next look for the printer in the lpoptions file...
@@ -1297,9 +1274,9 @@ recv_print_job(
 	  if (status)
 	    break;
 	}
-
-	fclose(fp);
       }
+
+      fclose(fp);
     }
   }
 
@@ -1725,5 +1702,5 @@ smart_gets(char *s,			/* I - Pointer to line buffer */
 
 
 /*
- * End of "$Id: cups-lpd.c 6333 2007-03-12 20:49:13Z mike $".
+ * End of "$Id: cups-lpd.c 7222 2008-01-16 22:20:33Z mike $".
  */
