@@ -1,25 +1,16 @@
 /*
- * "$Id: type.c,v 1.27 2005/01/03 19:29:59 mike Exp $"
+ * "$Id: type.c 7201 2008-01-09 04:03:00Z mike $"
  *
  *   MIME typing routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -27,7 +18,7 @@
  *   mimeAddTypeRule()  - Add a detection rule for a file type.
  *   mimeFileType()     - Determine the type of a file.
  *   mimeType()         - Lookup a file type.
- *   compare()          - Compare two MIME super/type names.
+ *   compare_types()    - Compare two MIME super/type names.
  *   checkrules()       - Check each rule in a list.
  *   patmatch()         - Pattern matching...
  */
@@ -46,37 +37,46 @@
 #include <cups/debug.h>
 
 
+/*
+ * Local types...
+ */
+
+typedef struct _mime_filebuf_s		/**** File buffer for MIME typing ****/
+{
+  cups_file_t	*fp;			/* File pointer */
+  int		offset,			/* Offset in file */
+		length;			/* Length of buffered data */
+  unsigned char	buffer[MIME_MAX_BUFFER];/* Buffered data */
+} _mime_filebuf_t;
+
+
 /*
  * Local functions...
  */
 
-static int	compare(mime_type_t **, mime_type_t **);
-static int	checkrules(const char *, cups_file_t *, mime_magic_t *);
-static int	patmatch(const char *, const char *);
+static int	compare_types(mime_type_t *t0, mime_type_t *t1);
+static int	checkrules(const char *filename, _mime_filebuf_t *fb,
+		           mime_magic_t *rules);
+static int	patmatch(const char *s, const char *pat);
 
 
 /*
  * 'mimeAddType()' - Add a MIME type to a database.
  */
 
-mime_type_t *			/* O - New (or existing) MIME type */
-mimeAddType(mime_t     *mime,	/* I - MIME database */
-            const char *super,	/* I - Super-type name */
-	    const char *type)	/* I - Type name */
+mime_type_t *				/* O - New (or existing) MIME type */
+mimeAddType(mime_t     *mime,		/* I - MIME database */
+            const char *super,		/* I - Super-type name */
+	    const char *type)		/* I - Type name */
 {
-  mime_type_t	*temp,		/* New MIME type */
-		**types;	/* New MIME types array */
+  mime_type_t	*temp;			/* New MIME type */
 
 
  /*
   * Range check input...
   */
 
-  if (mime == NULL || super == NULL || type == NULL)
-    return (NULL);
-
-  if (strlen(super) > (MIME_MAX_SUPER - 1) ||
-      strlen(type) > (MIME_MAX_TYPE - 1))
+  if (!mime || !super || !type)
     return (NULL);
 
  /*
@@ -90,35 +90,20 @@ mimeAddType(mime_t     *mime,	/* I - MIME database */
   * The type doesn't exist; add it...
   */
 
-  if ((temp = calloc(1, sizeof(mime_type_t))) == NULL)
+  if (!mime->types)
+    mime->types = cupsArrayNew((cups_array_func_t)compare_types, NULL);
+
+  if (!mime->types)
     return (NULL);
 
-  if (mime->num_types == 0)
-    types = (mime_type_t **)malloc(sizeof(mime_type_t *));
-  else
-    types = (mime_type_t **)realloc(mime->types, sizeof(mime_type_t *) * (mime->num_types + 1));
-
-  if (types == NULL)
-  {
-    free(temp);
+  if ((temp = calloc(1, sizeof(mime_type_t) - MIME_MAX_TYPE +
+                        strlen(type) + 1)) == NULL)
     return (NULL);
-  }
 
-  mime->types = types;
-  types += mime->num_types;
-  mime->num_types ++;
-
-  *types = temp;
   strlcpy(temp->super, super, sizeof(temp->super));
-  if ((temp->type = strdup(type)) == NULL)
-  {
-    mime->num_types --;
-    return (NULL);
-  }
+  strcpy(temp->type, type);		/* Safe: temp->type is allocated */
 
-  if (mime->num_types > 1)
-    qsort(mime->types, mime->num_types, sizeof(mime_type_t *),
-          (int (*)(const void *, const void *))compare);
+  cupsArrayAdd(mime->types, temp);
 
   return (temp);
 }
@@ -149,15 +134,15 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
   * Range check input...
   */
 
-  if (mt == NULL || rule == NULL)
+  if (!mt || !rule)
     return (-1);
 
  /*
   * Find the last rule in the top-level of the rules tree.
   */
 
-  for (current = mt->rules; current != NULL; current = current->next)
-    if (current->next == NULL)
+  for (current = mt->rules; current; current = current->next)
+    if (!current->next)
       break;
 
  /*
@@ -202,7 +187,8 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
     else if (*rule == '+' && current != NULL)
     {
       if (logic != MIME_MAGIC_AND &&
-          current != NULL && current->prev != NULL && current->prev->prev != NULL)
+          current != NULL && current->prev != NULL &&
+	  current->prev->prev != NULL)
       {
        /*
         * OK, we have more than 1 rule in the current tree level...  Make a
@@ -291,7 +277,8 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
       * Read an extension name or a function...
       */
 
-      for (ptr = name; isalnum(*rule & 255) && (ptr - name) < (sizeof(name) - 1);)
+      ptr = name;
+      while (isalnum(*rule & 255) && (ptr - name) < (sizeof(name) - 1))
         *ptr++ = *rule++;
 
       *ptr       = '\0';
@@ -389,25 +376,25 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
         * Figure out the function...
 	*/
 
-        if (strcmp(name, "match") == 0)
+        if (!strcmp(name, "match"))
 	  op = MIME_MAGIC_MATCH;
-	else if (strcmp(name, "ascii") == 0)
+	else if (!strcmp(name, "ascii"))
 	  op = MIME_MAGIC_ASCII;
-	else if (strcmp(name, "printable") == 0)
+	else if (!strcmp(name, "printable"))
 	  op = MIME_MAGIC_PRINTABLE;
-	else if (strcmp(name, "string") == 0)
+	else if (!strcmp(name, "string"))
 	  op = MIME_MAGIC_STRING;
-	else if (strcmp(name, "istring") == 0)
+	else if (!strcmp(name, "istring"))
 	  op = MIME_MAGIC_ISTRING;
-	else if (strcmp(name, "char") == 0)
+	else if (!strcmp(name, "char"))
 	  op = MIME_MAGIC_CHAR;
-	else if (strcmp(name, "short") == 0)
+	else if (!strcmp(name, "short"))
 	  op = MIME_MAGIC_SHORT;
-	else if (strcmp(name, "int") == 0)
+	else if (!strcmp(name, "int"))
 	  op = MIME_MAGIC_INT;
-	else if (strcmp(name, "locale") == 0)
+	else if (!strcmp(name, "locale"))
 	  op = MIME_MAGIC_LOCALE;
-	else if (strcmp(name, "contains") == 0)
+	else if (!strcmp(name, "contains"))
 	  op = MIME_MAGIC_CONTAINS;
 	else
 	  return (-1);
@@ -539,44 +526,62 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
 
 mime_type_t *				/* O - Type of file */
 mimeFileType(mime_t     *mime,		/* I - MIME database */
-             const char *pathname,	/* I - Name of file to check */
+             const char *pathname,	/* I - Name of file to check on disk */
+	     const char *filename,	/* I - Original filename or NULL */
 	     int        *compression)	/* O - Is the file compressed? */
 {
-  int		i;			/* Looping var */
-  cups_file_t	*fp;			/* File pointer */
-  mime_type_t	**types;		/* File types */
-  const char	*filename;		/* Base filename of file */
+  _mime_filebuf_t	fb;		/* File buffer */
+  const char		*base;		/* Base filename of file */
+  mime_type_t		*type;		/* File type */
 
 
+  DEBUG_printf(("mimeFileType(mime=%p, pathname=\"%s\", filename=\"%s\", "
+                "compression=%p)\n",
+                mime, pathname ? pathname : "(nil)",
+		filename ? filename : "(nil)",
+		compression));
+
  /*
   * Range check input parameters...
   */
 
-  if (mime == NULL || pathname == NULL)
+  if (!mime || !pathname)
     return (NULL);
 
  /*
   * Try to open the file...
   */
 
-  if ((fp = cupsFileOpen(pathname, "r")) == NULL)
+  if ((fb.fp = cupsFileOpen(pathname, "r")) == NULL)
     return (NULL);
 
+  fb.offset = -1;
+  fb.length = 0;
+
  /*
-  * Figure out the filename (without directory portion)...
+  * Figure out the base filename (without directory portion)...
   */
 
-  if ((filename = strrchr(pathname, '/')) != NULL)
-    filename ++;
+  if (filename)
+  {
+    if ((base = strrchr(filename, '/')) != NULL)
+      base ++;
+    else
+      base = filename;
+  }
+  else if ((base = strrchr(pathname, '/')) != NULL)
+    base ++;
   else
-    filename = pathname;
+    base = pathname;
 
  /*
   * Then check it against all known types...
   */
 
-  for (i = mime->num_types, types = mime->types; i > 0; i --, types ++)
-    if (checkrules(filename, fp, (*types)->rules))
+  for (type = (mime_type_t *)cupsArrayFirst(mime->types);
+       type;
+       type = (mime_type_t *)cupsArrayNext(mime->types))
+    if (checkrules(base, &fb, type->rules))
       break;
 
  /*
@@ -584,14 +589,11 @@ mimeFileType(mime_t     *mime,		/* I - MIME database */
   */
 
   if (compression)
-    *compression = cupsFileCompression(fp);
+    *compression = cupsFileCompression(fb.fp);
 
-  cupsFileClose(fp);
+  cupsFileClose(fb.fp);
 
-  if (i > 0)
-    return (*types);
-  else
-    return (NULL);
+  return (type);
 }
 
 
@@ -599,27 +601,19 @@ mimeFileType(mime_t     *mime,		/* I - MIME database */
  * 'mimeType()' - Lookup a file type.
  */
 
-mime_type_t *			/* O - Matching file type definition */
-mimeType(mime_t     *mime,	/* I - MIME database */
-         const char *super,	/* I - Super-type name */
-	 const char *type)	/* I - Type name */
+mime_type_t *				/* O - Matching file type definition */
+mimeType(mime_t     *mime,		/* I - MIME database */
+         const char *super,		/* I - Super-type name */
+	 const char *type)		/* I - Type name */
 {
-  mime_type_t	key,		/* MIME type search key*/
-		*keyptr,	/* Key pointer... */
-		**match;	/* Matching pointer */
+  mime_type_t	key;			/* MIME type search key*/
+
 
  /*
   * Range check input...
   */
 
-  if (mime == NULL || super == NULL || type == NULL)
-    return (NULL);
-
-  if (strlen(super) > (MIME_MAX_SUPER - 1) ||
-      strlen(type) > (MIME_MAX_TYPE - 1))
-    return (NULL);
-
-  if (mime->num_types == 0)
+  if (!mime || !super || !type)
     return (NULL);
 
  /*
@@ -627,34 +621,25 @@ mimeType(mime_t     *mime,	/* I - MIME database */
   */
 
   strlcpy(key.super, super, sizeof(key.super));
-  key.type = (char *)type;
+  strlcpy(key.type, type, sizeof(key.type));
 
-  keyptr = &key;
-
-  match = (mime_type_t **)bsearch(&keyptr, mime->types, mime->num_types,
-                                  sizeof(mime_type_t *),
-                                  (int (*)(const void *, const void *))compare);
-
-  if (match == NULL)
-    return (NULL);
-  else
-    return (*match);
+  return ((mime_type_t *)cupsArrayFind(mime->types, &key));
 }
 
 
 /*
- * 'compare()' - Compare two MIME super/type names.
+ * 'compare_types()' - Compare two MIME super/type names.
  */
 
-static int			/* O - Result of comparison */
-compare(mime_type_t **t0,	/* I - First type */
-        mime_type_t **t1)	/* I - Second type */
+static int				/* O - Result of comparison */
+compare_types(mime_type_t *t0,		/* I - First type */
+              mime_type_t *t1)		/* I - Second type */
 {
-  int	i;			/* Result of comparison */
+  int	i;				/* Result of comparison */
 
 
-  if ((i = strcasecmp((*t0)->super, (*t1)->super)) == 0)
-    i = strcasecmp((*t0)->type, (*t1)->type);
+  if ((i = strcasecmp(t0->super, t1->super)) == 0)
+    i = strcasecmp(t0->type, t1->type);
 
   return (i);
 }
@@ -665,9 +650,9 @@ compare(mime_type_t **t0,	/* I - First type */
  */
 
 static int				/* O - 1 if match, 0 if no match */
-checkrules(const char   *filename,	/* I - Filename */
-           cups_file_t  *fp,		/* I - File to check */
-           mime_magic_t *rules)		/* I - Rules to check */
+checkrules(const char      *filename,	/* I - Filename */
+           _mime_filebuf_t *fb,		/* I - File to check */
+           mime_magic_t    *rules)	/* I - Rules to check */
 {
   int		n;			/* Looping var */
   int		region;			/* Region to look at */
@@ -675,12 +660,30 @@ checkrules(const char   *filename,	/* I - Filename */
 		result,			/* Result of test */
 		intv;			/* Integer value */
   short		shortv;			/* Short value */
-  unsigned char	buffer[MIME_MAX_BUFFER],/* Input buffer */
-		*bufptr;		/* Current buffer position */
-  int		bufoffset,		/* Offset in file for buffer */
-		buflength;		/* Length of data in buffer */
+  unsigned char	*bufptr;		/* Pointer into buffer */
+#ifdef DEBUG
+  const char	* const debug_tests[] =	/* Test names... */
+		{
+		  "NOP",		/* No operation */
+		  "AND",		/* Logical AND of all children */
+		  "OR",			/* Logical OR of all children */
+		  "MATCH",		/* Filename match */
+		  "ASCII",		/* ASCII characters in range */
+		  "PRINTABLE",		/* Printable characters (32-255) */
+		  "STRING",		/* String matches */
+		  "CHAR",		/* Character/byte matches */
+		  "SHORT",		/* Short/16-bit word matches */
+		  "INT",		/* Integer/32-bit word matches */
+		  "LOCALE"		/* Current locale matches string */
+		  "CONTAINS"		/* File contains a string */
+		  "ISTRING"		/* Case-insensitive string matches */
+		};
+#endif /* DEBUG */
 
 
+  DEBUG_printf(("checkrules(filename=\"%s\", fb=%p, rules=%p)\n", filename,
+                fb, rules));
+
   if (rules == NULL)
     return (0);
 
@@ -689,9 +692,7 @@ checkrules(const char   *filename,	/* I - Filename */
   else
     logic = rules->parent->op;
 
-  bufoffset = -1;
-  buflength = 0;
-  result    = 0;
+  result = 0;
 
   while (rules != NULL)
   {
@@ -710,28 +711,29 @@ checkrules(const char   *filename,	/* I - Filename */
 	  * Load the buffer if necessary...
 	  */
 
-          if (bufoffset < 0 || rules->offset < bufoffset ||
-	      (rules->offset + rules->length) > (bufoffset + buflength))
+          if (fb->offset < 0 || rules->offset < fb->offset ||
+	      (rules->offset + rules->length) > (fb->offset + fb->length))
 	  {
 	   /*
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
-	    bufoffset = rules->offset;
+            cupsFileSeek(fb->fp, rules->offset);
+	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
+	                              sizeof(fb->buffer));
+	    fb->offset = rules->offset;
 	  }
 
          /*
 	  * Test for ASCII printable characters plus standard control chars.
 	  */
 
-	  if ((rules->offset + rules->length) > (bufoffset + buflength))
-	    n = bufoffset + buflength - rules->offset;
+	  if ((rules->offset + rules->length) > (fb->offset + fb->length))
+	    n = fb->offset + fb->length - rules->offset;
 	  else
 	    n = rules->length;
 
-          bufptr = buffer + rules->offset - bufoffset;
+          bufptr = fb->buffer + rules->offset - fb->offset;
 	  while (n > 0)
 	    if ((*bufptr >= 32 && *bufptr <= 126) ||
 	        (*bufptr >= 8 && *bufptr <= 13) ||
@@ -751,28 +753,29 @@ checkrules(const char   *filename,	/* I - Filename */
 	  * Load the buffer if necessary...
 	  */
 
-          if (bufoffset < 0 || rules->offset < bufoffset ||
-	      (rules->offset + rules->length) > (bufoffset + buflength))
+          if (fb->offset < 0 || rules->offset < fb->offset ||
+	      (rules->offset + rules->length) > (fb->offset + fb->length))
 	  {
 	   /*
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
-	    bufoffset = rules->offset;
+            cupsFileSeek(fb->fp, rules->offset);
+	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
+	                              sizeof(fb->buffer));
+	    fb->offset = rules->offset;
 	  }
 
          /*
 	  * Test for 8-bit printable characters plus standard control chars.
 	  */
 
-	  if ((rules->offset + rules->length) > (bufoffset + buflength))
-	    n = bufoffset + buflength - rules->offset;
+	  if ((rules->offset + rules->length) > (fb->offset + fb->length))
+	    n = fb->offset + fb->length - rules->offset;
 	  else
 	    n = rules->length;
 
-          bufptr = buffer + rules->offset - bufoffset;
+          bufptr = fb->buffer + rules->offset - fb->offset;
 
 	  while (n > 0)
 	    if (*bufptr >= 128 ||
@@ -790,20 +793,29 @@ checkrules(const char   *filename,	/* I - Filename */
 	  break;
 
       case MIME_MAGIC_STRING :
+          DEBUG_printf(("    string(%d, \"%s\")\n", rules->offset,
+	                rules->value.stringv));
+
          /*
 	  * Load the buffer if necessary...
 	  */
 
-          if (bufoffset < 0 || rules->offset < bufoffset ||
-	      (rules->offset + rules->length) > (bufoffset + buflength))
+          if (fb->offset < 0 || rules->offset < fb->offset ||
+	      (rules->offset + rules->length) > (fb->offset + fb->length))
 	  {
 	   /*
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
-	    bufoffset = rules->offset;
+            cupsFileSeek(fb->fp, rules->offset);
+	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
+	                              sizeof(fb->buffer));
+	    fb->offset = rules->offset;
+
+            DEBUG_printf(("        loaded %d byte fb->buffer at %d, starts "
+	                  "with \"%c%c%c%c\"...\n",
+	                  fb->length, fb->offset, fb->buffer[0], fb->buffer[1],
+			  fb->buffer[2], fb->buffer[3]));
 	  }
 
          /*
@@ -811,11 +823,12 @@ checkrules(const char   *filename,	/* I - Filename */
 	  * short then don't compare - it can't match...
 	  */
 
-	  if ((rules->offset + rules->length) > (bufoffset + buflength))
+	  if ((rules->offset + rules->length) > (fb->offset + fb->length))
 	    result = 0;
 	  else
-            result = (memcmp(buffer + rules->offset - bufoffset,
+            result = (memcmp(fb->buffer + rules->offset - fb->offset,
 	                     rules->value.stringv, rules->length) == 0);
+          DEBUG_printf(("    result=%d\n", result));
 	  break;
 
       case MIME_MAGIC_ISTRING :
@@ -823,16 +836,17 @@ checkrules(const char   *filename,	/* I - Filename */
 	  * Load the buffer if necessary...
 	  */
 
-          if (bufoffset < 0 || rules->offset < bufoffset ||
-	      (rules->offset + rules->length) > (bufoffset + buflength))
+          if (fb->offset < 0 || rules->offset < fb->offset ||
+	      (rules->offset + rules->length) > (fb->offset + fb->length))
 	  {
 	   /*
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
-	    bufoffset = rules->offset;
+            cupsFileSeek(fb->fp, rules->offset);
+	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
+	                              sizeof(fb->buffer));
+	    fb->offset = rules->offset;
 	  }
 
          /*
@@ -840,10 +854,11 @@ checkrules(const char   *filename,	/* I - Filename */
 	  * short then don't compare - it can't match...
 	  */
 
-	  if ((rules->offset + rules->length) > (bufoffset + buflength))
+	  if ((rules->offset + rules->length) > (fb->offset + fb->length))
 	    result = 0;
 	  else
-            result = (strncasecmp((char *)buffer + rules->offset - bufoffset,
+            result = (strncasecmp((char *)fb->buffer + rules->offset -
+	                              fb->offset,
 	                          rules->value.stringv, rules->length) == 0);
 	  break;
 
@@ -852,15 +867,16 @@ checkrules(const char   *filename,	/* I - Filename */
 	  * Load the buffer if necessary...
 	  */
 
-          if (bufoffset < 0 || rules->offset < bufoffset)
+          if (fb->offset < 0 || rules->offset < fb->offset)
 	  {
 	   /*
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
-	    bufoffset = rules->offset;
+            cupsFileSeek(fb->fp, rules->offset);
+	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
+	                              sizeof(fb->buffer));
+	    fb->offset = rules->offset;
 	  }
 
 	 /*
@@ -868,10 +884,11 @@ checkrules(const char   *filename,	/* I - Filename */
 	  * can't match...
 	  */
 
-	  if (buflength < 1)
+	  if (fb->length < 1)
 	    result = 0;
 	  else
-	    result = (buffer[rules->offset - bufoffset] == rules->value.charv);
+	    result = (fb->buffer[rules->offset - fb->offset] ==
+	                  rules->value.charv);
 	  break;
 
       case MIME_MAGIC_SHORT :
@@ -879,16 +896,17 @@ checkrules(const char   *filename,	/* I - Filename */
 	  * Load the buffer if necessary...
 	  */
 
-          if (bufoffset < 0 || rules->offset < bufoffset ||
-	      (rules->offset + 2) > (bufoffset + buflength))
+          if (fb->offset < 0 || rules->offset < fb->offset ||
+	      (rules->offset + 2) > (fb->offset + fb->length))
 	  {
 	   /*
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
-	    bufoffset = rules->offset;
+            cupsFileSeek(fb->fp, rules->offset);
+	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
+	                              sizeof(fb->buffer));
+	    fb->offset = rules->offset;
 	  }
 
 	 /*
@@ -896,11 +914,11 @@ checkrules(const char   *filename,	/* I - Filename */
 	  * can't match...
 	  */
 
-	  if (buflength < 2)
+	  if (fb->length < 2)
 	    result = 0;
 	  else
 	  {
-	    bufptr = buffer + rules->offset - bufoffset;
+	    bufptr = fb->buffer + rules->offset - fb->offset;
 	    shortv = (bufptr[0] << 8) | bufptr[1];
 	    result = (shortv == rules->value.shortv);
 	  }
@@ -911,16 +929,17 @@ checkrules(const char   *filename,	/* I - Filename */
 	  * Load the buffer if necessary...
 	  */
 
-          if (bufoffset < 0 || rules->offset < bufoffset ||
-	      (rules->offset + 4) > (bufoffset + buflength))
+          if (fb->offset < 0 || rules->offset < fb->offset ||
+	      (rules->offset + 4) > (fb->offset + fb->length))
 	  {
 	   /*
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
-	    bufoffset = rules->offset;
+            cupsFileSeek(fb->fp, rules->offset);
+	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
+	                              sizeof(fb->buffer));
+	    fb->offset = rules->offset;
 	  }
 
 	 /*
@@ -928,22 +947,24 @@ checkrules(const char   *filename,	/* I - Filename */
 	  * can't match...
 	  */
 
-	  if (buflength < 4)
+	  if (fb->length < 4)
 	    result = 0;
 	  else
 	  {
-	    bufptr = buffer + rules->offset - bufoffset;
-	    intv   = (((((bufptr[0] << 8) | bufptr[1]) << 8) | bufptr[2]) << 8) |
-	             bufptr[3];;
+	    bufptr = fb->buffer + rules->offset - fb->offset;
+	    intv   = (((((bufptr[0] << 8) | bufptr[1]) << 8) |
+	               bufptr[2]) << 8) | bufptr[3];
 	    result = (intv == rules->value.intv);
 	  }
 	  break;
 
       case MIME_MAGIC_LOCALE :
 #if defined(WIN32) || defined(__EMX__) || defined(__APPLE__)
-          result = (strcmp(rules->value.localev, setlocale(LC_ALL, "")) == 0);
+          result = (strcmp(rules->value.localev,
+	                   setlocale(LC_ALL, "")) == 0);
 #else
-          result = (strcmp(rules->value.localev, setlocale(LC_MESSAGES, "")) == 0);
+          result = (strcmp(rules->value.localev,
+	                   setlocale(LC_MESSAGES, "")) == 0);
 #endif /* __APPLE__ */
 	  break;
 
@@ -952,16 +973,17 @@ checkrules(const char   *filename,	/* I - Filename */
 	  * Load the buffer if necessary...
 	  */
 
-          if (bufoffset < 0 || rules->offset < bufoffset ||
-	      (rules->offset + rules->region) > (bufoffset + buflength))
+          if (fb->offset < 0 || rules->offset < fb->offset ||
+	      (rules->offset + rules->region) > (fb->offset + fb->length))
 	  {
 	   /*
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
-	    bufoffset = rules->offset;
+            cupsFileSeek(fb->fp, rules->offset);
+	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
+	                              sizeof(fb->buffer));
+	    fb->offset = rules->offset;
 	  }
 
          /*
@@ -969,25 +991,26 @@ checkrules(const char   *filename,	/* I - Filename */
 	  * short then don't compare - it can't match...
 	  */
 
-	  if ((rules->offset + rules->length) > (bufoffset + buflength))
+	  if ((rules->offset + rules->length) > (fb->offset + fb->length))
 	    result = 0;
 	  else
 	  {
-	    if (buflength > rules->region)
+	    if (fb->length > rules->region)
 	      region = rules->region - rules->length;
 	    else
-	      region = buflength - rules->length;
+	      region = fb->length - rules->length;
 
 	    for (n = 0; n < region; n ++)
-	      if ((result = (memcmp(buffer + rules->offset - bufoffset + n,
-	                            rules->value.stringv, rules->length) == 0)) != 0)
+	      if ((result = (memcmp(fb->buffer + rules->offset - fb->offset + n,
+	                            rules->value.stringv,
+				    rules->length) == 0)) != 0)
 		break;
           }
 	  break;
 
       default :
           if (rules->child != NULL)
-	    result = checkrules(filename, fp, rules->child);
+	    result = checkrules(filename, fb, rules->child);
 	  else
 	    result = 0;
 	  break;
@@ -1006,7 +1029,8 @@ checkrules(const char   *filename,	/* I - Filename */
     * the the rule set is false...
     */
 
-    DEBUG_printf(("result of test %p is %d\n", rules, result));
+    DEBUG_printf(("    result of test %p (MIME_MAGIC_%s) is %d\n", rules,
+                  debug_tests[rules->op], result));
 
     if ((result && logic == MIME_MAGIC_OR) ||
         (!result && logic == MIME_MAGIC_AND))
@@ -1053,7 +1077,7 @@ patmatch(const char *s,		/* I - String to match against */
 
       pat ++;
       if (*pat == '\0')
-        return (1);	/* Last pattern char is *, so everything matches now... */
+        return (1);	/* Last pattern char is *, so everything matches... */
 
      /*
       * Test all remaining combinations until we get to the end of the string.
@@ -1119,8 +1143,8 @@ patmatch(const char *s,		/* I - String to match against */
   }
 
  /*
-  * Done parsing the pattern and string; return 1 if the last character matches
-  * and 0 otherwise...
+  * Done parsing the pattern and string; return 1 if the last character
+  * matches and 0 otherwise...
   */
 
   return (*s == *pat);
@@ -1128,5 +1152,5 @@ patmatch(const char *s,		/* I - String to match against */
 
 
 /*
- * End of "$Id: type.c,v 1.27 2005/01/03 19:29:59 mike Exp $".
+ * End of "$Id: type.c 7201 2008-01-09 04:03:00Z mike $".
  */
