@@ -1,25 +1,16 @@
 /*
- * "$Id: cupsaddsmb.c 6625 2007-07-06 22:39:54Z mike $"
+ * "$Id: cupsaddsmb.c 7037 2007-10-19 02:39:57Z mike $"
  *
  *   "cupsaddsmb" command for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 2001-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -161,8 +152,13 @@ main(int  argc,				/* I - Number of command-line arguments */
       }
 
       if (SAMBAServer == NULL)
+      {
 	SAMBAServer = cupsServer();
 
+	if (SAMBAServer[0] == '/')	/* Use localhost instead of domain socket */
+	  SAMBAServer = "localhost";
+      }
+
       if ((status = export_dest(http, argv[i])) != 0)
 	return (status);
     }
@@ -191,8 +187,13 @@ main(int  argc,				/* I - Number of command-line arguments */
     */
 
     if (SAMBAServer == NULL)
+    {
       SAMBAServer = cupsServer();
 
+      if (SAMBAServer[0] == '/')	/* Use localhost instead of domain socket */
+	SAMBAServer = "localhost";
+    }
+
     num_dests = cupsGetDests2(http, &dests);
 
     for (j = 0, status = 0; j < num_dests; j ++)
@@ -297,5 +298,5 @@ usage(void)
 
 
 /*
- * End of "$Id: cupsaddsmb.c 6625 2007-07-06 22:39:54Z mike $".
+ * End of "$Id: cupsaddsmb.c 7037 2007-10-19 02:39:57Z mike $".
  */
