@@ -1,25 +1,16 @@
 /*
- * "$Id: lpstat.c 6090 2006-11-14 16:35:27Z mike $"
+ * "$Id: lpstat.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   "lpstat" command for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -42,6 +33,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <cups/http-private.h>
 #include <cups/string.h>
 #include <cups/cups.h>
 #include <cups/i18n.h>
@@ -1396,9 +1388,11 @@ show_jobs(http_t     *http,		/* I - HTTP connection to server */
           int        ranking,		/* I - Show job ranking? */
 	  const char *which)		/* I - Show which jobs? */
 {
+  int		i;			/* Looping var */
   ipp_t		*request,		/* IPP Request */
 		*response;		/* IPP Response */
-  ipp_attribute_t *attr;		/* Current attribute */
+  ipp_attribute_t *attr,		/* Current attribute */
+		*reasons;		/* Job state reasons attribute */
   const char	*dest,			/* Pointer into job-printer-uri */
 		*username,		/* Pointer to job-originating-user-name */
 		*title;			/* Pointer to job-name */
@@ -1419,7 +1413,8 @@ show_jobs(http_t     *http,		/* I - HTTP connection to server */
 		  "job-name",
 		  "time-at-creation",
 		  "job-printer-uri",
-		  "job-originating-user-name"
+		  "job-originating-user-name",
+		  "job-state-reasons"
 		};
 
 
@@ -1494,6 +1489,7 @@ show_jobs(http_t     *http,		/* I - HTTP connection to server */
       dest     = NULL;
       jobtime  = 0;
       title    = "no title";
+      reasons  = NULL;
 
       while (attr != NULL && attr->group_tag == IPP_TAG_JOB)
       {
@@ -1522,6 +1518,10 @@ show_jobs(http_t     *http,		/* I - HTTP connection to server */
 	    attr->value_tag == IPP_TAG_NAME)
 	  title = attr->values[0].string.text;
 
+        if (!strcmp(attr->name, "job-state-reasons") &&
+	    attr->value_tag == IPP_TAG_KEYWORD)
+	  reasons = attr;
+
         attr = attr->next;
       }
 
@@ -1664,7 +1664,17 @@ show_jobs(http_t     *http,		/* I - HTTP connection to server */
 	                    temp, username ? username : "unknown",
 			    1024.0 * size, date);
           if (long_status)
+          {
+	    if (reasons)
+	    {
+	      _cupsLangPuts(stdout, _("\tAlerts:"));
+	      for (i = 0; i < reasons->num_values; i ++)
+		_cupsLangPrintf(stdout, " %s",
+				reasons->values[i].string.text);
+	      _cupsLangPuts(stdout, "\n");
+	    }
 	    _cupsLangPrintf(stdout, _("\tqueued for %s\n"), dest);
+	  }
 	}
       }
 
@@ -2248,5 +2258,5 @@ show_scheduler(http_t *http)	/* I - HTTP connection to server */
 
 
 /*
- * End of "$Id: lpstat.c 6090 2006-11-14 16:35:27Z mike $".
+ * End of "$Id: lpstat.c 6649 2007-07-11 21:46:42Z mike $".
  */
